# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test the functionality of server interceptors."""

import asyncio
import functools
import logging
from typing import Any, Awaitable, Callable, Tuple
import unittest

import grpc
from grpc.experimental import aio
from grpc.experimental import wrap_server_method_handler

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import TEST_CONTEXT_VAR
from tests_aio.unit._test_server import start_test_server

_NUM_STREAM_RESPONSES = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_PAYLOAD_SIZE = 42


class _LoggingInterceptor(aio.ServerInterceptor):
    def __init__(self, tag: str, record: list) -> None:
        self.tag = tag
        self.record = record

    async def intercept_service(
        self,
        continuation: Callable[
            [grpc.HandlerCallDetails], Awaitable[grpc.RpcMethodHandler]
        ],
        handler_call_details: grpc.HandlerCallDetails,
    ) -> grpc.RpcMethodHandler:
        self.record.append(
            self.tag + ":" + TEST_CONTEXT_VAR.get("intercept_service")
        )
        return await continuation(handler_call_details)


class _ContextVarSettingInterceptor(aio.ServerInterceptor):
    def __init__(self, value: str) -> None:
        self.value = value

    async def intercept_service(
        self,
        continuation: Callable[
            [grpc.HandlerCallDetails], Awaitable[grpc.RpcMethodHandler]
        ],
        handler_call_details: grpc.HandlerCallDetails,
    ) -> grpc.RpcMethodHandler:
        new_value = self.value
        old_value = TEST_CONTEXT_VAR.get("")
        if old_value:
            new_value = old_value + ":" + new_value
        TEST_CONTEXT_VAR.set(new_value)
        return await continuation(handler_call_details)


class _GenericInterceptor(aio.ServerInterceptor):
    def __init__(
        self,
        fn: Callable[
            [
                Callable[
                    [grpc.HandlerCallDetails], Awaitable[grpc.RpcMethodHandler]
                ],
                grpc.HandlerCallDetails,
            ],
            Any,
        ],
    ) -> None:
        self._fn = fn

    async def intercept_service(
        self,
        continuation: Callable[
            [grpc.HandlerCallDetails], Awaitable[grpc.RpcMethodHandler]
        ],
        handler_call_details: grpc.HandlerCallDetails,
    ) -> grpc.RpcMethodHandler:
        return await self._fn(continuation, handler_call_details)


def _filter_server_interceptor(
    condition: Callable, interceptor: aio.ServerInterceptor
) -> aio.ServerInterceptor:
    async def intercept_service(
        continuation: Callable[
            [grpc.HandlerCallDetails], Awaitable[grpc.RpcMethodHandler]
        ],
        handler_call_details: grpc.HandlerCallDetails,
    ) -> grpc.RpcMethodHandler:
        if condition(handler_call_details):
            return await interceptor.intercept_service(
                continuation, handler_call_details
            )
        return await continuation(handler_call_details)

    return _GenericInterceptor(intercept_service)


class _CacheInterceptor(aio.ServerInterceptor):
    """An interceptor that caches response based on request message."""

    def __init__(self, cache_store=None):
        self.cache_store = cache_store or {}

    async def intercept_service(
        self,
        continuation: Callable[
            [grpc.HandlerCallDetails], Awaitable[grpc.RpcMethodHandler]
        ],
        handler_call_details: grpc.HandlerCallDetails,
    ) -> grpc.RpcMethodHandler:
        # Get the actual handler
        handler = await continuation(handler_call_details)

        # Only intercept unary call RPCs
        if handler and (
            handler.request_streaming
            or handler.response_streaming  # pytype: disable=attribute-error
        ):  # pytype: disable=attribute-error
            return handler

        def wrapper(
            behavior: Callable[
                [messages_pb2.SimpleRequest, aio.ServicerContext],
                messages_pb2.SimpleResponse,
            ],
        ):
            @functools.wraps(behavior)
            async def wrapper(
                request: messages_pb2.SimpleRequest,
                context: aio.ServicerContext,
            ) -> messages_pb2.SimpleResponse:
                if request.response_size not in self.cache_store:
                    self.cache_store[request.response_size] = await behavior(
                        request, context
                    )
                return self.cache_store[request.response_size]

            return wrapper

        return wrap_server_method_handler(wrapper, handler)


async def _create_server_stub_pair(
    record: list, *interceptors: aio.ServerInterceptor
) -> Tuple[aio.Server, test_pb2_grpc.TestServiceStub]:
    """Creates a server-stub pair with given record and interceptors.

    Returning the server object to protect it from being garbage collected.
    """
    server_target, server = await start_test_server(
        interceptors=interceptors, record=record
    )
    channel = aio.insecure_channel(server_target)
    return server, test_pb2_grpc.TestServiceStub(channel)


class TestServerInterceptor(AioTestBase):
    async def test_invalid_interceptor(self):
        class InvalidInterceptor:
            """Just an invalid Interceptor"""

        with self.assertRaises(ValueError):
            server_target, _ = await start_test_server(
                interceptors=(InvalidInterceptor(),)
            )

    async def test_executed_right_order(self):
        record = []
        server_target, _ = await start_test_server(
            record=record,
            interceptors=(
                _LoggingInterceptor("log1", record),
                _ContextVarSettingInterceptor("context_var_value"),
                _LoggingInterceptor("log2", record),
            ),
        )

        async with aio.insecure_channel(server_target) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())
            response = await call

            # Check that all interceptors were executed--and were executed
            # in the right order--before calling the servicer.
            self.assertSequenceEqual(
                [
                    "log1:intercept_service",
                    "log2:context_var_value",
                    "servicer:context_var_value",
                ],
                record,
            )
            self.assertIsInstance(response, messages_pb2.SimpleResponse)

    async def test_unique_context_per_call(self):
        record = []
        server, stub = await _create_server_stub_pair(
            record,
            _LoggingInterceptor("log1", record),
            _ContextVarSettingInterceptor("context_var_value"),
            _LoggingInterceptor("log2", record),
        )

        response = await stub.UnaryCall(
            messages_pb2.SimpleRequest(response_size=42)
        )

        self.assertSequenceEqual(
            [
                "log1:intercept_service",
                "log2:context_var_value",
                "servicer:context_var_value",
            ],
            record,
        )
        record.clear()

        response = await stub.UnaryCall(
            messages_pb2.SimpleRequest(response_size=42)
        )

        self.assertSequenceEqual(
            [
                "log1:intercept_service",
                "log2:context_var_value",
                "servicer:context_var_value",
            ],
            record,
        )

    async def test_response_ok(self):
        record = []
        server_target, _ = await start_test_server(
            interceptors=(_LoggingInterceptor("log1", record),)
        )

        async with aio.insecure_channel(server_target) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())
            response = await call
            code = await call.code()

            self.assertSequenceEqual(["log1:intercept_service"], record)
            self.assertIsInstance(response, messages_pb2.SimpleResponse)
            self.assertEqual(code, grpc.StatusCode.OK)

    async def test_apply_different_interceptors_by_metadata(self):
        record = []
        conditional_interceptor = _filter_server_interceptor(
            lambda x: ("secret", "42") in x.invocation_metadata,
            _LoggingInterceptor("log3", record),
        )
        server_target, _ = await start_test_server(
            interceptors=(
                _LoggingInterceptor("log1", record),
                conditional_interceptor,
                _LoggingInterceptor("log2", record),
            )
        )

        async with aio.insecure_channel(server_target) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            metadata = aio.Metadata(
                ("key", "value"),
            )
            call = multicallable(
                messages_pb2.SimpleRequest(), metadata=metadata
            )
            await call
            self.assertSequenceEqual(
                [
                    "log1:intercept_service",
                    "log2:intercept_service",
                ],
                record,
            )

            record.clear()
            metadata = aio.Metadata(("key", "value"), ("secret", "42"))
            call = multicallable(
                messages_pb2.SimpleRequest(), metadata=metadata
            )
            await call
            self.assertSequenceEqual(
                [
                    "log1:intercept_service",
                    "log3:intercept_service",
                    "log2:intercept_service",
                ],
                record,
            )

    async def test_response_caching(self):
        # Prepares a preset value to help testing
        interceptor = _CacheInterceptor(
            {
                42: messages_pb2.SimpleResponse(
                    payload=messages_pb2.Payload(body=b"\x42")
                )
            }
        )

        # Constructs a server with the cache interceptor
        server, stub = await _create_server_stub_pair([], interceptor)

        # Tests if the cache store is used
        response = await stub.UnaryCall(
            messages_pb2.SimpleRequest(response_size=42)
        )
        self.assertEqual(1, len(interceptor.cache_store[42].payload.body))
        self.assertEqual(interceptor.cache_store[42], response)

        # Tests response can be cached
        response = await stub.UnaryCall(
            messages_pb2.SimpleRequest(response_size=1337)
        )
        self.assertEqual(1337, len(interceptor.cache_store[1337].payload.body))
        self.assertEqual(interceptor.cache_store[1337], response)
        response = await stub.UnaryCall(
            messages_pb2.SimpleRequest(response_size=1337)
        )
        self.assertEqual(interceptor.cache_store[1337], response)

    async def test_interceptor_unary_stream(self):
        record = []
        server, stub = await _create_server_stub_pair(
            record,
            _ContextVarSettingInterceptor("context_var_value"),
            _LoggingInterceptor("log_unary_stream", record),
        )

        # Prepares the request
        request = messages_pb2.StreamingOutputCallRequest()
        for _ in range(_NUM_STREAM_RESPONSES):
            request.response_parameters.append(
                messages_pb2.ResponseParameters(
                    size=_RESPONSE_PAYLOAD_SIZE,
                )
            )

        # Tests if the cache store is used
        call = stub.StreamingOutputCall(request)

        # Ensures the RPC goes fine
        async for response in call:
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))
        self.assertEqual(await call.code(), grpc.StatusCode.OK)

        self.assertSequenceEqual(
            [
                "log_unary_stream:context_var_value",
                "servicer:context_var_value",
            ],
            record,
        )

    async def test_interceptor_stream_unary(self):
        record = []
        server, stub = await _create_server_stub_pair(
            record,
            _ContextVarSettingInterceptor("context_var_value"),
            _LoggingInterceptor("log_stream_unary", record),
        )

        # Invokes the actual RPC
        call = stub.StreamingInputCall()

        # Prepares the request
        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        # Sends out requests
        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(request)
        await call.done_writing()

        # Validates the responses
        response = await call
        self.assertIsInstance(response, messages_pb2.StreamingInputCallResponse)
        self.assertEqual(
            _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
            response.aggregated_payload_size,
        )

        self.assertEqual(await call.code(), grpc.StatusCode.OK)

        self.assertSequenceEqual(
            [
                "log_stream_unary:context_var_value",
                "servicer:context_var_value",
            ],
            record,
        )

    async def test_interceptor_stream_stream(self):
        record = []
        server, stub = await _create_server_stub_pair(
            record,
            _ContextVarSettingInterceptor("context_var_value"),
            _LoggingInterceptor("log_stream_stream", record),
        )

        # Prepares the request
        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        async def gen():
            for _ in range(_NUM_STREAM_RESPONSES):
                yield request

        # Invokes the actual RPC
        call = stub.StreamingInputCall(gen())

        # Validates the responses
        response = await call
        self.assertIsInstance(response, messages_pb2.StreamingInputCallResponse)
        self.assertEqual(
            _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
            response.aggregated_payload_size,
        )

        self.assertEqual(await call.code(), grpc.StatusCode.OK)

        self.assertSequenceEqual(
            [
                "log_stream_stream:context_var_value",
                "servicer:context_var_value",
            ],
            record,
        )


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/postgres_proxy/v3alpha/postgres_proxy.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/postgres_proxy/v3alpha/postgres_proxy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nTcontrib/envoy/extensions/filters/network/postgres_proxy/v3alpha/postgres_proxy.proto\x12\x37\x65nvoy.extensions.filters.network.postgres_proxy.v3alpha\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x87\x03\n\rPostgresProxy\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x36\n\x12\x65nable_sql_parsing\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\"\n\rterminate_ssl\x18\x03 \x01(\x08\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x64\n\x0cupstream_ssl\x18\x04 \x01(\x0e\x32N.envoy.extensions.filters.network.postgres_proxy.v3alpha.PostgresProxy.SSLMode\x12\x66\n\x0e\x64ownstream_ssl\x18\x05 \x01(\x0e\x32N.envoy.extensions.filters.network.postgres_proxy.v3alpha.PostgresProxy.SSLMode\".\n\x07SSLMode\x12\x0b\n\x07\x44ISABLE\x10\x00\x12\x0b\n\x07REQUIRE\x10\x01\x12\t\n\x05\x41LLOW\x10\x02\x42\xcf\x01\nEio.envoyproxy.envoy.extensions.filters.network.postgres_proxy.v3alphaB\x12PostgresProxyProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/postgres_proxy/v3alpha\xba\x80\xc8\xd1\x06\x04\x08\x01\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.postgres_proxy.v3alpha.postgres_proxy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nEio.envoyproxy.envoy.extensions.filters.network.postgres_proxy.v3alphaB\022PostgresProxyProtoP\001Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/postgres_proxy/v3alpha\272\200\310\321\006\004\010\001\020\002'
  _globals['_POSTGRESPROXY'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_POSTGRESPROXY'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_POSTGRESPROXY'].fields_by_name['terminate_ssl']._loaded_options = None
  _globals['_POSTGRESPROXY'].fields_by_name['terminate_ssl']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_POSTGRESPROXY']._serialized_start=271
  _globals['_POSTGRESPROXY']._serialized_end=662
  _globals['_POSTGRESPROXY_SSLMODE']._serialized_start=616
  _globals['_POSTGRESPROXY_SSLMODE']._serialized_end=662
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Buildgen .proto files list plugin.

This parses the list of targets from the yaml build file, and creates
a list called "protos" that contains all of the proto file names.

"""

import re


def mako_plugin(dictionary):
    """The exported plugin code for list_protos.

    Some projects generators may want to get the full list of unique .proto files
    that are being included in a project. This code extracts all files referenced
    in any library or target that ends in .proto, and builds and exports that as
    a list called "protos".

    """

    libs = dictionary.get("libs", [])
    targets = dictionary.get("targets", [])

    proto_re = re.compile("(.*)\\.proto")

    protos = set()
    for lib in libs:
        for src in lib.get("src", []):
            m = proto_re.match(src)
            if m:
                protos.add(m.group(1))
    for tgt in targets:
        for src in tgt.get("src", []):
            m = proto_re.match(src)
            if m:
                protos.add(m.group(1))

    protos = sorted(protos)

    dictionary["protos"] = protos



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Defines behavior for WHEN clients send requests.

Each client exposes a non-blocking send_request() method that the
ClientRunner invokes either periodically or in response to some event.
"""

import abc
import threading
import time


class ClientRunner:
    """Abstract interface for sending requests from clients."""

    __metaclass__ = abc.ABCMeta

    def __init__(self, client):
        self._client = client

    @abc.abstractmethod
    def start(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def stop(self):
        raise NotImplementedError()


class OpenLoopClientRunner(ClientRunner):
    def __init__(self, client, interval_generator):
        super(OpenLoopClientRunner, self).__init__(client)
        self._is_running = False
        self._interval_generator = interval_generator
        self._dispatch_thread = threading.Thread(
            target=self._dispatch_requests, args=()
        )

    def start(self):
        self._is_running = True
        self._client.start()
        self._dispatch_thread.start()

    def stop(self):
        self._is_running = False
        self._client.stop()
        self._dispatch_thread.join()
        self._client = None

    def _dispatch_requests(self):
        while self._is_running:
            self._client.send_request()
            time.sleep(next(self._interval_generator))


class ClosedLoopClientRunner(ClientRunner):
    def __init__(self, client, request_count, no_ping_pong):
        super(ClosedLoopClientRunner, self).__init__(client)
        self._is_running = False
        self._request_count = request_count
        # For server-streaming RPC, don't spawn new RPC after each response.
        # This yield at most ~17% for single RPC scenarios.
        if not no_ping_pong:
            # Send a new request on each response for closed loop
            self._client.add_response_callback(self._send_request)

    def start(self):
        self._is_running = True
        self._client.start()
        for _ in range(self._request_count):
            self._client.send_request()

    def stop(self):
        self._is_running = False
        self._client.stop()
        self._client = None

    def _send_request(self, client, unused_response_time):
        if self._is_running:
            client.send_request()



#!/usr/bin/env python2.7

# Copyright 2019 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import yaml

BUILDS_YAML_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "preprocessed_builds.yaml"
)
with open(BUILDS_YAML_PATH) as f:
    builds = yaml.safe_load(f)

for build in builds:
    build["build"] = "private"
    build["build_system"] = []
    build["language"] = "c"
    build["secure"] = False
print(yaml.dump({"libs": builds}))



# Copyright 2023 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An example of setting a server connection timeout independent from the
overall RPC timeout.

For stream server, if client set wait_for_ready but server never actually starts,
client will wait indefinitely, this example will do the following steps to set a
timeout on client side:
1. Set server to return initial_metadata once it receives request.
2. Client will set a timer (customized client timeout) waiting for initial_metadata.
3. Client will timeout if it didn't receive initial_metadata.
"""

from concurrent import futures
import logging
from time import sleep

import grpc

helloworld_pb2, helloworld_pb2_grpc = grpc.protos_and_services(
    "helloworld.proto"
)

_INITIAL_METADATA = ((b"initial-md", "initial-md-value"),)


def starting_up_server():
    print("sleeping 5s before sending metadata back")
    sleep(5)


def do_work():
    print("server is processing the request")
    sleep(5)


class Greeter(helloworld_pb2_grpc.GreeterServicer):
    def SayHelloStreamReply(self, request, servicer_context):
        # Suppose server will take some time to setup, client can set the time it willing to wait
        # for server to up and running.
        starting_up_server()

        # Initial metadata will be sent back immediately after calling send_initial_metadata.
        print("sending initial metadata back")
        servicer_context.send_initial_metadata(_INITIAL_METADATA)

        # Time for server to process the request.
        do_work()

        # Sending actual response.
        for i in range(3):
            yield helloworld_pb2.HelloReply(
                message="Hello %s times %s" % (request.name, i)
            )


def serve():
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    helloworld_pb2_grpc.add_GreeterServicer_to_server(Greeter(), server)
    server.add_insecure_port("[::]:50051")
    print("starting server")
    server.start()
    server.wait_for_termination()


if __name__ == "__main__":
    logging.basicConfig()
    serve()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/router/cluster_specifier/golang/v3alpha/golang.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/router/cluster_specifier/golang/v3alpha/golang.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nMcontrib/envoy/extensions/router/cluster_specifier/golang/v3alpha/golang.proto\x12\x38\x65nvoy.extensions.router.cluster_specifier.golang.v3alpha\x1a\x19google/protobuf/any.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x8c\x01\n\x06\x43onfig\x12\x1b\n\nlibrary_id\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x1d\n\x0clibrary_path\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12 \n\x0f\x64\x65\x66\x61ult_cluster\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12$\n\x06\x63onfig\x18\x04 \x01(\x0b\x32\x14.google.protobuf.AnyB\xd0\x01\nFio.envoyproxy.envoy.extensions.router.cluster_specifier.golang.v3alphaB\x0bGolangProtoP\x01Zggithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/router/cluster_specifier/golang/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.router.cluster_specifier.golang.v3alpha.golang_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nFio.envoyproxy.envoy.extensions.router.cluster_specifier.golang.v3alphaB\013GolangProtoP\001Zggithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/router/cluster_specifier/golang/v3alpha\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_CONFIG'].fields_by_name['library_id']._loaded_options = None
  _globals['_CONFIG'].fields_by_name['library_id']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG'].fields_by_name['library_path']._loaded_options = None
  _globals['_CONFIG'].fields_by_name['library_path']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG'].fields_by_name['default_cluster']._loaded_options = None
  _globals['_CONFIG'].fields_by_name['default_cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG']._serialized_start=256
  _globals['_CONFIG']._serialized_end=396
# @@protoc_insertion_point(module_scope)



# Copyright 2021 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import collections
from concurrent import futures
import logging
import signal
import socket
import sys
import threading
import time
from typing import DefaultDict, Dict, List, Mapping, Sequence, Set, Tuple

import grpc
from grpc_channelz.v1 import channelz
from grpc_channelz.v1 import channelz_pb2
from grpc_csm_observability import CsmOpenTelemetryPlugin
from grpc_health.v1 import health as grpc_health
from grpc_health.v1 import health_pb2
from grpc_health.v1 import health_pb2_grpc
from grpc_reflection.v1alpha import reflection
from opentelemetry.exporter.prometheus import PrometheusMetricReader
from opentelemetry.sdk.metrics import MeterProvider
from prometheus_client import start_http_server

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2
from src.proto.grpc.testing import test_pb2_grpc
from src.python.grpcio_tests.tests.fork import native_debug

native_debug.install_failure_signal_handler()

# NOTE: This interop server is not fully compatible with all xDS interop tests.
#  It currently only implements enough functionality to pass the xDS security
#  tests.

_LISTEN_HOST = "0.0.0.0"
_PROMETHEUS_PORT = 9464

_THREAD_POOL_SIZE = 256

logger = logging.getLogger()
console_handler = logging.StreamHandler()
formatter = logging.Formatter(fmt="%(asctime)s: %(levelname)-8s %(message)s")
console_handler.setFormatter(formatter)
logger.addHandler(console_handler)


class TestService(test_pb2_grpc.TestServiceServicer):
    def __init__(self, server_id, hostname):
        self._server_id = server_id
        self._hostname = hostname

    def EmptyCall(
        self, _: empty_pb2.Empty, context: grpc.ServicerContext
    ) -> empty_pb2.Empty:
        context.send_initial_metadata((("hostname", self._hostname),))
        return empty_pb2.Empty()

    def UnaryCall(
        self, request: messages_pb2.SimpleRequest, context: grpc.ServicerContext
    ) -> messages_pb2.SimpleResponse:
        context.send_initial_metadata((("hostname", self._hostname),))
        if request.response_size > 0:
            response = messages_pb2.SimpleResponse(
                payload=messages_pb2.Payload(body=b"0" * request.response_size)
            )
        else:
            response = messages_pb2.SimpleResponse()
        response.server_id = self._server_id
        response.hostname = self._hostname
        return response


def _configure_maintenance_server(
    server: grpc.Server, maintenance_port: int
) -> None:
    channelz.add_channelz_servicer(server)
    listen_address = f"{_LISTEN_HOST}:{maintenance_port}"
    server.add_insecure_port(listen_address)
    health_servicer = grpc_health.HealthServicer(
        experimental_non_blocking=True,
        experimental_thread_pool=futures.ThreadPoolExecutor(
            max_workers=_THREAD_POOL_SIZE
        ),
    )

    health_pb2_grpc.add_HealthServicer_to_server(health_servicer, server)
    SERVICE_NAMES = (
        test_pb2.DESCRIPTOR.services_by_name["TestService"].full_name,
        health_pb2.DESCRIPTOR.services_by_name["Health"].full_name,
        channelz_pb2.DESCRIPTOR.services_by_name["Channelz"].full_name,
        reflection.SERVICE_NAME,
    )
    for service in SERVICE_NAMES:
        health_servicer.set(service, health_pb2.HealthCheckResponse.SERVING)
    reflection.enable_server_reflection(SERVICE_NAMES, server)


def _configure_test_server(
    server: grpc.Server, port: int, secure_mode: bool, server_id: str
) -> None:
    test_pb2_grpc.add_TestServiceServicer_to_server(
        TestService(server_id, socket.gethostname()), server
    )
    listen_address = f"{_LISTEN_HOST}:{port}"
    if not secure_mode:
        server.add_insecure_port(listen_address)
    else:
        logger.info("Running with xDS Server credentials")
        server_fallback_creds = grpc.insecure_server_credentials()
        server_creds = grpc.xds_server_credentials(server_fallback_creds)
        server.add_secure_port(listen_address, server_creds)


def _run(
    port: int,
    maintenance_port: int,
    secure_mode: bool,
    server_id: str,
    enable_csm_observability: bool,
) -> None:
    csm_plugin = None
    if enable_csm_observability:
        csm_plugin = _prepare_csm_observability_plugin()
        csm_plugin.register_global()
    if port == maintenance_port:
        server = grpc.server(
            futures.ThreadPoolExecutor(max_workers=_THREAD_POOL_SIZE)
        )
        _configure_test_server(server, port, secure_mode, server_id)
        _configure_maintenance_server(server, maintenance_port)
        server.start()
        logger.info("Test server listening on port %d", port)
        logger.info("Maintenance server listening on port %d", maintenance_port)
        server.wait_for_termination()
    else:
        maintenance_server = grpc.server(
            futures.ThreadPoolExecutor(max_workers=_THREAD_POOL_SIZE)
        )
        _configure_maintenance_server(maintenance_server, maintenance_port)
        maintenance_server.start()
        logger.info("Maintenance server listening on port %d", maintenance_port)
        test_server = grpc.server(
            futures.ThreadPoolExecutor(max_workers=_THREAD_POOL_SIZE),
            xds=secure_mode,
        )
        _configure_test_server(test_server, port, secure_mode, server_id)
        test_server.start()
        logger.info("Test server listening on port %d", port)
        test_server.wait_for_termination()
        maintenance_server.wait_for_termination()
    if csm_plugin:
        csm_plugin.deregister_global()


def bool_arg(arg: str) -> bool:
    if arg.lower() in ("true", "yes", "y"):
        return True
    elif arg.lower() in ("false", "no", "n"):
        return False
    else:
        raise argparse.ArgumentTypeError(f"Could not parse '{arg}' as a bool.")


def _prepare_csm_observability_plugin() -> CsmOpenTelemetryPlugin:
    # Start Prometheus client
    start_http_server(port=_PROMETHEUS_PORT, addr="0.0.0.0")
    reader = PrometheusMetricReader()
    meter_provider = MeterProvider(metric_readers=[reader])
    csm_plugin = CsmOpenTelemetryPlugin(
        meter_provider=meter_provider,
    )
    return csm_plugin


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Run Python xDS interop server."
    )
    parser.add_argument(
        "--port", type=int, default=8080, help="Port for test server."
    )
    parser.add_argument(
        "--maintenance_port",
        type=int,
        default=8080,
        help="Port for servers besides test server.",
    )
    parser.add_argument(
        "--secure_mode",
        type=bool_arg,
        default="False",
        help="If specified, uses xDS to retrieve server credentials.",
    )
    parser.add_argument(
        "--server_id",
        type=str,
        default="python_server",
        help="The server ID to return in responses..",
    )
    parser.add_argument(
        "--verbose",
        help="verbose log output",
        default=False,
        action="store_true",
    )
    parser.add_argument(
        "--enable_csm_observability",
        help="Whether to enable CSM Observability",
        default="False",
        type=bool_arg,
    )
    args = parser.parse_args()
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)
    if args.secure_mode and args.port == args.maintenance_port:
        raise ValueError(
            "--port and --maintenance_port must not be the same when"
            " --secure_mode is set."
        )
    _run(
        args.port,
        args.maintenance_port,
        args.secure_mode,
        args.server_id,
        args.enable_csm_observability,
    )



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Defines a number of module-scope gRPC scenarios to test clean exit."""

import argparse
import logging
import threading
import time

import grpc

from tests.unit.framework.common import test_constants

WAIT_TIME = 1000

REQUEST = b"request"

UNSTARTED_SERVER = "unstarted_server"
RUNNING_SERVER = "running_server"
POLL_CONNECTIVITY_NO_SERVER = "poll_connectivity_no_server"
POLL_CONNECTIVITY = "poll_connectivity"
IN_FLIGHT_UNARY_UNARY_CALL = "in_flight_unary_unary_call"
IN_FLIGHT_UNARY_STREAM_CALL = "in_flight_unary_stream_call"
IN_FLIGHT_STREAM_UNARY_CALL = "in_flight_stream_unary_call"
IN_FLIGHT_STREAM_STREAM_CALL = "in_flight_stream_stream_call"
IN_FLIGHT_PARTIAL_UNARY_STREAM_CALL = "in_flight_partial_unary_stream_call"
IN_FLIGHT_PARTIAL_STREAM_UNARY_CALL = "in_flight_partial_stream_unary_call"
IN_FLIGHT_PARTIAL_STREAM_STREAM_CALL = "in_flight_partial_stream_stream_call"

_SERVICE_NAME = "test"
UNARY_UNARY = b"UnaryUnary"
UNARY_STREAM = b"UnaryStream"
STREAM_UNARY = b"StreamUnary"
STREAM_STREAM = b"StreamStream"
PARTIAL_UNARY_STREAM = b"PartialUnaryStream"
PARTIAL_STREAM_UNARY = b"PartialStreamUnary"
PARTIAL_STREAM_STREAM = b"PartialStreamStream"

TEST_TO_METHOD = {
    IN_FLIGHT_UNARY_UNARY_CALL: UNARY_UNARY,
    IN_FLIGHT_UNARY_STREAM_CALL: UNARY_STREAM,
    IN_FLIGHT_STREAM_UNARY_CALL: STREAM_UNARY,
    IN_FLIGHT_STREAM_STREAM_CALL: STREAM_STREAM,
    IN_FLIGHT_PARTIAL_UNARY_STREAM_CALL: PARTIAL_UNARY_STREAM,
    IN_FLIGHT_PARTIAL_STREAM_UNARY_CALL: PARTIAL_STREAM_UNARY,
    IN_FLIGHT_PARTIAL_STREAM_STREAM_CALL: PARTIAL_STREAM_STREAM,
}


def hang_unary_unary(request, servicer_context):
    time.sleep(WAIT_TIME)


def hang_unary_stream(request, servicer_context):
    time.sleep(WAIT_TIME)


def hang_partial_unary_stream(request, servicer_context):
    for _ in range(test_constants.STREAM_LENGTH // 2):
        yield request
    time.sleep(WAIT_TIME)


def hang_stream_unary(request_iterator, servicer_context):
    time.sleep(WAIT_TIME)


def hang_partial_stream_unary(request_iterator, servicer_context):
    for _ in range(test_constants.STREAM_LENGTH // 2):
        next(request_iterator)
    time.sleep(WAIT_TIME)


def hang_stream_stream(request_iterator, servicer_context):
    time.sleep(WAIT_TIME)


def hang_partial_stream_stream(request_iterator, servicer_context):
    for _ in range(test_constants.STREAM_LENGTH // 2):
        yield next(request_iterator)  # pylint: disable=stop-iteration-return
    time.sleep(WAIT_TIME)


class MethodHandler(grpc.RpcMethodHandler):
    def __init__(self, request_streaming, response_streaming, partial_hang):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None
        if self.request_streaming and self.response_streaming:
            if partial_hang:
                self.stream_stream = hang_partial_stream_stream
            else:
                self.stream_stream = hang_stream_stream
        elif self.request_streaming:
            if partial_hang:
                self.stream_unary = hang_partial_stream_unary
            else:
                self.stream_unary = hang_stream_unary
        elif self.response_streaming:
            if partial_hang:
                self.unary_stream = hang_partial_unary_stream
            else:
                self.unary_stream = hang_unary_stream
        else:
            self.unary_unary = hang_unary_unary


class GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        if handler_call_details.method == UNARY_UNARY:
            return MethodHandler(False, False, False)
        elif handler_call_details.method == UNARY_STREAM:
            return MethodHandler(False, True, False)
        elif handler_call_details.method == STREAM_UNARY:
            return MethodHandler(True, False, False)
        elif handler_call_details.method == STREAM_STREAM:
            return MethodHandler(True, True, False)
        elif handler_call_details.method == PARTIAL_UNARY_STREAM:
            return MethodHandler(False, True, True)
        elif handler_call_details.method == PARTIAL_STREAM_UNARY:
            return MethodHandler(True, False, True)
        elif handler_call_details.method == PARTIAL_STREAM_STREAM:
            return MethodHandler(True, True, True)
        else:
            return None


_METHOD_HANDLERS = {
    UNARY_UNARY: MethodHandler(False, False, False),
    UNARY_STREAM: MethodHandler(False, True, False),
    STREAM_UNARY: MethodHandler(True, False, False),
    STREAM_STREAM: MethodHandler(True, True, False),
    PARTIAL_UNARY_STREAM: MethodHandler(False, True, True),
    PARTIAL_STREAM_UNARY: MethodHandler(True, False, True),
    PARTIAL_STREAM_STREAM: MethodHandler(True, True, True),
}


# Traditional executors will not exit until all their
# current jobs complete.  Because we submit jobs that will
# never finish, we don't want to block exit on these jobs.
class DaemonPool:
    def submit(self, fn, *args, **kwargs):
        thread = threading.Thread(target=fn, args=args, kwargs=kwargs)
        thread.daemon = True
        thread.start()

    def shutdown(self, wait=True):
        pass


def infinite_request_iterator():
    while True:
        yield REQUEST


if __name__ == "__main__":
    logging.basicConfig()
    parser = argparse.ArgumentParser()
    parser.add_argument("scenario", type=str)
    parser.add_argument(
        "--wait_for_interrupt", dest="wait_for_interrupt", action="store_true"
    )
    args = parser.parse_args()

    if args.scenario == UNSTARTED_SERVER:
        server = grpc.server(DaemonPool(), options=(("grpc.so_reuseport", 0),))
        if args.wait_for_interrupt:
            time.sleep(WAIT_TIME)
    elif args.scenario == RUNNING_SERVER:
        server = grpc.server(DaemonPool(), options=(("grpc.so_reuseport", 0),))
        port = server.add_insecure_port("[::]:0")
        server.start()
        if args.wait_for_interrupt:
            time.sleep(WAIT_TIME)
    elif args.scenario == POLL_CONNECTIVITY_NO_SERVER:
        channel = grpc.insecure_channel("localhost:12345")

        def connectivity_callback(connectivity):
            pass

        channel.subscribe(connectivity_callback, try_to_connect=True)
        if args.wait_for_interrupt:
            time.sleep(WAIT_TIME)
    elif args.scenario == POLL_CONNECTIVITY:
        server = grpc.server(DaemonPool(), options=(("grpc.so_reuseport", 0),))
        port = server.add_insecure_port("[::]:0")
        server.start()
        channel = grpc.insecure_channel("localhost:%d" % port)

        def connectivity_callback(connectivity):
            pass

        channel.subscribe(connectivity_callback, try_to_connect=True)
        if args.wait_for_interrupt:
            time.sleep(WAIT_TIME)

    else:
        server = grpc.server(DaemonPool(), options=(("grpc.so_reuseport", 0),))
        port = server.add_insecure_port("[::]:0")
        server.add_registered_method_handlers(_SERVICE_NAME, _METHOD_HANDLERS)
        server.start()
        channel = grpc.insecure_channel("localhost:%d" % port)

        method = (
            grpc._common.fully_qualified_method(
                _SERVICE_NAME, TEST_TO_METHOD[args.scenario]
            ),
        )

        if args.scenario == IN_FLIGHT_UNARY_UNARY_CALL:
            multi_callable = channel.unary_unary(
                method,
                _registered_method=True,
            )
            future = multi_callable.future(REQUEST)
            result, call = multi_callable.with_call(REQUEST)
        elif (
            args.scenario == IN_FLIGHT_UNARY_STREAM_CALL
            or args.scenario == IN_FLIGHT_PARTIAL_UNARY_STREAM_CALL
        ):
            multi_callable = channel.unary_stream(
                method,
                _registered_method=True,
            )
            response_iterator = multi_callable(REQUEST)
            for response in response_iterator:
                pass
        elif (
            args.scenario == IN_FLIGHT_STREAM_UNARY_CALL
            or args.scenario == IN_FLIGHT_PARTIAL_STREAM_UNARY_CALL
        ):
            multi_callable = channel.stream_unary(
                method,
                _registered_method=True,
            )
            future = multi_callable.future(infinite_request_iterator())
            result, call = multi_callable.with_call(
                iter([REQUEST] * test_constants.STREAM_LENGTH)
            )
        elif (
            args.scenario == IN_FLIGHT_STREAM_STREAM_CALL
            or args.scenario == IN_FLIGHT_PARTIAL_STREAM_STREAM_CALL
        ):
            multi_callable = channel.stream_stream(
                method,
                _registered_method=True,
            )
            response_iterator = multi_callable(infinite_request_iterator())
            for response in response_iterator:
                pass



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import defaultdict
import datetime
import logging
import os
import sys
import time
from typing import Any, Callable, Dict, List, Optional, Set
import unittest

import grpc
import grpc_observability
from grpc_observability import _open_telemetry_measures
from grpc_observability._open_telemetry_observability import (
    GRPC_OTHER_LABEL_VALUE,
)
from grpc_observability._open_telemetry_observability import GRPC_METHOD_LABEL
from grpc_observability._open_telemetry_observability import GRPC_TARGET_LABEL
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import AggregationTemporality
from opentelemetry.sdk.metrics.export import MetricExportResult
from opentelemetry.sdk.metrics.export import MetricExporter
from opentelemetry.sdk.metrics.export import MetricsData
from opentelemetry.sdk.metrics.export import PeriodicExportingMetricReader

from tests.observability import _test_server

logger = logging.getLogger(__name__)

STREAM_LENGTH = 5
OTEL_EXPORT_INTERVAL_S = 0.5


class OTelMetricExporter(MetricExporter):
    """Implementation of :class:`MetricExporter` that export metrics to the
    provided metric_list.

    all_metrics: A dict which key is grpc_observability._opentelemetry_measures.Metric.name,
        value is a list of labels recorded for that metric.
        An example item of this dict:
            {"grpc.client.attempt.started":
              [{'grpc.method': 'test/UnaryUnary', 'grpc.target': 'localhost:42517'},
               {'grpc.method': 'other', 'grpc.target': 'localhost:42517'}]}
    """

    def __init__(
        self,
        all_metrics: Dict[str, List],
        preferred_temporality: Dict[type, AggregationTemporality] = None,
        preferred_aggregation: Dict[
            type, "opentelemetry.sdk.metrics.view.Aggregation"
        ] = None,
    ):
        super().__init__(
            preferred_temporality=preferred_temporality,
            preferred_aggregation=preferred_aggregation,
        )
        self.all_metrics = all_metrics

    def export(
        self,
        metrics_data: MetricsData,
        timeout_millis: float = 10_000,
        **kwargs,
    ) -> MetricExportResult:
        self.record_metric(metrics_data)
        return MetricExportResult.SUCCESS

    def shutdown(self, timeout_millis: float = 30_000, **kwargs) -> None:
        pass

    def force_flush(self, timeout_millis: float = 10_000) -> bool:
        return True

    def record_metric(self, metrics_data: MetricsData) -> None:
        for resource_metric in metrics_data.resource_metrics:
            for scope_metric in resource_metric.scope_metrics:
                for metric in scope_metric.metrics:
                    for data_point in metric.data.data_points:
                        self.all_metrics[metric.name].append(
                            data_point.attributes
                        )


class _ClientUnaryUnaryInterceptor(grpc.UnaryUnaryClientInterceptor):
    def intercept_unary_unary(
        self, continuation, client_call_details, request_or_iterator
    ):
        response = continuation(client_call_details, request_or_iterator)
        return response


class _ServerInterceptor(grpc.ServerInterceptor):
    def intercept_service(self, continuation, handler_call_details):
        return continuation(handler_call_details)


@unittest.skipIf(
    os.name == "nt" or "darwin" in sys.platform,
    "Observability is not supported in Windows and MacOS",
)
class OpenTelemetryObservabilityTest(unittest.TestCase):
    def setUp(self):
        self.all_metrics = defaultdict(list)
        otel_exporter = OTelMetricExporter(self.all_metrics)
        reader = PeriodicExportingMetricReader(
            exporter=otel_exporter,
            export_interval_millis=OTEL_EXPORT_INTERVAL_S * 1000,
        )
        self._provider = MeterProvider(metric_readers=[reader])
        self._server = None
        self._port = None

    def tearDown(self):
        if self._server:
            self._server.stop(0)

    def testRecordUnaryUnaryUseContextManager(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server()
            self._server = server
            _test_server.unary_unary_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

    def testRecordUnaryUnaryUseGlobalInit(self):
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        )
        otel_plugin.register_global()

        server, port = _test_server.start_server()
        self._server = server
        _test_server.unary_unary_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())
        otel_plugin.deregister_global()

    def testCallGlobalInitThrowErrorWhenGlobalCalled(self):
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        )
        otel_plugin.register_global()
        try:
            otel_plugin.register_global()
        except RuntimeError as exp:
            self.assertIn(
                "gPRC Python observability was already initialized", str(exp)
            )

        otel_plugin.deregister_global()

    def testCallGlobalInitThrowErrorWhenContextManagerCalled(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            try:
                otel_plugin = grpc_observability.OpenTelemetryPlugin(
                    meter_provider=self._provider
                )
                otel_plugin.register_global()
            except RuntimeError as exp:
                self.assertIn(
                    "gPRC Python observability was already initialized",
                    str(exp),
                )

    def testCallContextManagerThrowErrorWhenGlobalInitCalled(self):
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        )
        otel_plugin.register_global()
        try:
            with grpc_observability.OpenTelemetryPlugin(
                meter_provider=self._provider
            ):
                pass
        except RuntimeError as exp:
            self.assertIn(
                "gPRC Python observability was already initialized", str(exp)
            )
        otel_plugin.deregister_global()

    def testContextManagerThrowErrorWhenContextManagerCalled(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            try:
                with grpc_observability.OpenTelemetryPlugin(
                    meter_provider=self._provider
                ):
                    pass
            except RuntimeError as exp:
                self.assertIn(
                    "gPRC Python observability was already initialized",
                    str(exp),
                )

    def testNoErrorCallGlobalInitThenContextManager(self):
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        )
        otel_plugin.register_global()
        otel_plugin.deregister_global()

        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            pass

    def testNoErrorCallContextManagerThenGlobalInit(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            pass
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        )
        otel_plugin.register_global()
        otel_plugin.deregister_global()

    def testRecordUnaryUnaryWithClientInterceptor(self):
        interceptor = _ClientUnaryUnaryInterceptor()
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server()
            self._server = server
            _test_server.intercepted_unary_unary_call(
                port=port, interceptors=interceptor
            )

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

    def testRecordUnaryUnaryWithServerInterceptor(self):
        interceptor = _ServerInterceptor()
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server(interceptors=[interceptor])
            self._server = server
            _test_server.unary_unary_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

    def testRecordUnaryUnaryClientOnly(self):
        server, port = _test_server.start_server()
        self._server = server

        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            _test_server.unary_unary_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_client_metrics_names(self.all_metrics)

    def testNoRecordBeforeInit(self):
        server, port = _test_server.start_server()
        _test_server.unary_unary_call(port=port)
        self.assertEqual(len(self.all_metrics), 0)
        server.stop(0)

        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server()
            self._server = server
            _test_server.unary_unary_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

    def testNoRecordAfterExitUseContextManager(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server()
            self._server = server
            self._port = port
            _test_server.unary_unary_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

        self.all_metrics = defaultdict(list)
        _test_server.unary_unary_call(port=self._port)
        with self.assertRaisesRegex(AssertionError, "No metrics was exported"):
            self._validate_metrics_exist(self.all_metrics)

    def testNoRecordAfterExitUseGlobal(self):
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        )
        otel_plugin.register_global()

        server, port = _test_server.start_server()
        self._server = server
        self._port = port
        _test_server.unary_unary_call(port=port)
        otel_plugin.deregister_global()

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

        self.all_metrics = defaultdict(list)
        _test_server.unary_unary_call(port=self._port)
        with self.assertRaisesRegex(AssertionError, "No metrics was exported"):
            self._validate_metrics_exist(self.all_metrics)

    def testRecordUnaryStream(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server()
            self._server = server
            _test_server.unary_stream_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

    def testRecordStreamUnary(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server()
            self._server = server
            _test_server.stream_unary_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

    def testRecordStreamStream(self):
        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server()
            self._server = server
            _test_server.stream_stream_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())

    def testTargetAttributeFilter(self):
        main_server, main_port = _test_server.start_server()
        backup_server, backup_port = _test_server.start_server()
        main_target = f"localhost:{main_port}"
        backup_target = f"localhost:{backup_port}"

        # Replace target label with 'other' for main_server.
        def target_filter(target: str) -> bool:
            if main_target in target:
                return False
            return True

        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider, target_attribute_filter=target_filter
        ):
            _test_server.unary_unary_call(port=main_port)
            _test_server.unary_unary_call(port=backup_port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_client_metrics_names(self.all_metrics)

        target_values = set()
        for label_list in self.all_metrics.values():
            for labels in label_list:
                if GRPC_TARGET_LABEL in labels:
                    target_values.add(labels[GRPC_TARGET_LABEL])
        self.assertTrue(GRPC_OTHER_LABEL_VALUE in target_values)
        self.assertTrue(backup_target in target_values)

        main_server.stop(0)
        backup_server.stop(0)

    def testMethodAttributeFilter(self):
        # method_filter should replace method name 'test/UnaryUnaryFiltered' with 'other'.
        FILTERED_METHOD_NAME = "test/UnaryUnaryFiltered"

        def method_filter(method: str) -> bool:
            if FILTERED_METHOD_NAME in method:
                return False
            return True

        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider,
            generic_method_attribute_filter=method_filter,
        ):
            server, port = _test_server.start_server(register_method=False)
            self._server = server
            _test_server.unary_unary_call(port=port, registered_method=True)
            _test_server.unary_unary_filtered_call(port=port)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())
        method_values = set()
        for label_list in self.all_metrics.values():
            for labels in label_list:
                if GRPC_METHOD_LABEL in labels:
                    method_values.add(labels[GRPC_METHOD_LABEL])
        self.assertTrue(GRPC_OTHER_LABEL_VALUE in method_values)
        self.assertTrue(FILTERED_METHOD_NAME not in method_values)

    def testClientNonRegisteredMethod(self):
        UNARY_METHOD_NAME = "test/UnaryUnary"

        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server(register_method=True)
            self._server = server
            _test_server.unary_unary_call(port=port, registered_method=False)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())
        client_method_values = set()
        server_method_values = set()
        for metric_name, label_list in self.all_metrics.items():
            for labels in label_list:
                if GRPC_METHOD_LABEL in labels:
                    if "grpc.client" in metric_name:
                        client_method_values.add(labels[GRPC_METHOD_LABEL])
                    elif "grpc.server" in metric_name:
                        server_method_values.add(labels[GRPC_METHOD_LABEL])
        # For client metrics, all method name should be replaced with 'other'.
        self.assertTrue(GRPC_OTHER_LABEL_VALUE in client_method_values)
        self.assertTrue(UNARY_METHOD_NAME not in client_method_values)

        # For server metrics, all method name should be 'test/UnaryUnary'.
        self.assertTrue(GRPC_OTHER_LABEL_VALUE not in server_method_values)
        self.assertTrue(UNARY_METHOD_NAME in server_method_values)

    def testServerNonRegisteredMethod(self):
        UNARY_METHOD_NAME = "test/UnaryUnary"

        with grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider
        ):
            server, port = _test_server.start_server(register_method=False)
            self._server = server
            _test_server.unary_unary_call(port=port, registered_method=True)

        self._validate_metrics_exist(self.all_metrics)
        self._validate_all_metrics_names(self.all_metrics.keys())
        client_method_values = set()
        server_method_values = set()
        for metric_name, label_list in self.all_metrics.items():
            for labels in label_list:
                if GRPC_METHOD_LABEL in labels:
                    if "grpc.client" in metric_name:
                        client_method_values.add(labels[GRPC_METHOD_LABEL])
                    elif "grpc.server" in metric_name:
                        server_method_values.add(labels[GRPC_METHOD_LABEL])
        # For client metrics, all method name should be 'test/UnaryUnary'.
        self.assertTrue(GRPC_OTHER_LABEL_VALUE not in client_method_values)
        self.assertTrue(UNARY_METHOD_NAME in client_method_values)

        # For server metrics, all method name should be replaced with 'other'.
        self.assertTrue(GRPC_OTHER_LABEL_VALUE in server_method_values)
        self.assertTrue(UNARY_METHOD_NAME not in server_method_values)

    def assert_eventually(
        self,
        predicate: Callable[[], bool],
        *,
        timeout: Optional[datetime.timedelta] = None,
        message: Optional[Callable[[], str]] = None,
    ) -> None:
        message = message or (lambda: "Proposition did not evaluate to true")
        timeout = timeout or datetime.timedelta(seconds=5)
        end = datetime.datetime.now() + timeout
        while datetime.datetime.now() < end:
            if predicate():
                break
            time.sleep(0.5)
        else:
            self.fail(message() + " after " + str(timeout))

    def _validate_metrics_exist(self, all_metrics: Dict[str, Any]) -> None:
        # Sleep here to make sure we have at least one export from OTel MetricExporter.
        self.assert_eventually(
            lambda: len(all_metrics.keys()) > 1,
            message=lambda: f"No metrics was exported",
        )

    def _validate_all_metrics_names(self, metric_names: Set[str]) -> None:
        self._validate_server_metrics_names(metric_names)
        self._validate_client_metrics_names(metric_names)

    def _validate_server_metrics_names(self, metric_names: Set[str]) -> None:
        for base_metric in _open_telemetry_measures.base_metrics():
            if "grpc.server" in base_metric.name:
                self.assertTrue(
                    base_metric.name in metric_names,
                    msg=f"metric {base_metric.name} not found in exported metrics: {metric_names}!",
                )

    def _validate_client_metrics_names(self, metric_names: Set[str]) -> None:
        for base_metric in _open_telemetry_measures.base_metrics():
            if "grpc.client" in base_metric.name:
                self.assertTrue(
                    base_metric.name in metric_names,
                    msg=f"metric {base_metric.name} not found in exported metrics: {metric_names}!",
                )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common interfaces and implementation."""

import abc
import collections


def _fuss(tuplified_metadata):
    return tuplified_metadata + (
        (
            "grpc.metadata_added_by_runtime",
            "gRPC is allowed to add metadata in transmission and does so.",
        ),
    )


FUSSED_EMPTY_METADATA = _fuss(())


def fuss_with_metadata(metadata):
    if metadata is None:
        return FUSSED_EMPTY_METADATA
    return _fuss(tuple(metadata))


def rpc_names(service_descriptors):
    rpc_names_to_descriptors = {}
    for service_descriptor in service_descriptors:
        for method_descriptor in service_descriptor.methods_by_name.values():
            rpc_name = "/{}/{}".format(
                service_descriptor.full_name, method_descriptor.name
            )
            rpc_names_to_descriptors[rpc_name] = method_descriptor
    return rpc_names_to_descriptors


class ChannelRpcRead(
    collections.namedtuple(
        "ChannelRpcRead",
        (
            "response",
            "trailing_metadata",
            "code",
            "details",
        ),
    )
):
    pass


class ChannelRpcHandler(abc.ABC):
    @abc.abstractmethod
    def initial_metadata(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def add_request(self, request):
        raise NotImplementedError()

    @abc.abstractmethod
    def close_requests(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def take_response(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def cancel(self, code, details):
        raise NotImplementedError()

    @abc.abstractmethod
    def termination(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def is_active(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def time_remaining(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def add_callback(self, callback):
        raise NotImplementedError()


class ChannelHandler(abc.ABC):
    @abc.abstractmethod
    def invoke_rpc(
        self,
        method_full_rpc_name,
        invocation_metadata,
        requests,
        requests_closed,
        timeout,
    ):
        raise NotImplementedError()


class ServerRpcRead(
    collections.namedtuple(
        "ServerRpcRead",
        (
            "request",
            "requests_closed",
            "terminated",
        ),
    )
):
    pass


REQUESTS_CLOSED = ServerRpcRead(None, True, False)
TERMINATED = ServerRpcRead(None, False, True)


class ServerRpcHandler(abc.ABC):
    @abc.abstractmethod
    def send_initial_metadata(self, initial_metadata):
        raise NotImplementedError()

    @abc.abstractmethod
    def take_request(self):
        raise NotImplementedError()

    @abc.abstractmethod
    def add_response(self, response):
        raise NotImplementedError()

    @abc.abstractmethod
    def send_termination(self, trailing_metadata, code, details):
        raise NotImplementedError()

    @abc.abstractmethod
    def add_termination_callback(self, callback):
        raise NotImplementedError()


class Serverish(abc.ABC):
    @abc.abstractmethod
    def invoke_unary_unary(
        self, method_descriptor, handler, invocation_metadata, request, deadline
    ):
        raise NotImplementedError()

    @abc.abstractmethod
    def invoke_unary_stream(
        self, method_descriptor, handler, invocation_metadata, request, deadline
    ):
        raise NotImplementedError()

    @abc.abstractmethod
    def invoke_stream_unary(
        self, method_descriptor, handler, invocation_metadata, deadline
    ):
        raise NotImplementedError()

    @abc.abstractmethod
    def invoke_stream_stream(
        self, method_descriptor, handler, invocation_metadata, deadline
    ):
        raise NotImplementedError()



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python client used to test negative http2 conditions."""

import argparse
import time

import grpc

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc


def _validate_payload_type_and_length(response, expected_type, expected_length):
    if response.payload.type is not expected_type:
        raise ValueError(
            "expected payload type %s, got %s"
            % (expected_type, type(response.payload.type))
        )
    elif len(response.payload.body) != expected_length:
        raise ValueError(
            "expected payload body size %d, got %d"
            % (expected_length, len(response.payload.body))
        )


def _expect_status_code(call, expected_code):
    if call.code() != expected_code:
        raise ValueError(
            "expected code %s, got %s" % (expected_code, call.code())
        )


def _expect_status_details(call, expected_details):
    if call.details() != expected_details:
        raise ValueError(
            "expected message %s, got %s" % (expected_details, call.details())
        )


def _validate_status_code_and_details(call, expected_code, expected_details):
    _expect_status_code(call, expected_code)
    _expect_status_details(call, expected_details)


# common requests
_REQUEST_SIZE = 314159
_RESPONSE_SIZE = 271828

_SIMPLE_REQUEST = messages_pb2.SimpleRequest(
    response_type=messages_pb2.COMPRESSABLE,
    response_size=_RESPONSE_SIZE,
    payload=messages_pb2.Payload(body=b"\x00" * _REQUEST_SIZE),
)


def _goaway(stub):
    first_response = stub.UnaryCall(_SIMPLE_REQUEST)
    _validate_payload_type_and_length(
        first_response, messages_pb2.COMPRESSABLE, _RESPONSE_SIZE
    )
    time.sleep(1)
    second_response = stub.UnaryCall(_SIMPLE_REQUEST)
    _validate_payload_type_and_length(
        second_response, messages_pb2.COMPRESSABLE, _RESPONSE_SIZE
    )


def _rst_after_header(stub):
    resp_future = stub.UnaryCall.future(_SIMPLE_REQUEST)
    _validate_status_code_and_details(
        resp_future,
        grpc.StatusCode.INTERNAL,
        "Received RST_STREAM with error code 0",
    )


def _rst_during_data(stub):
    resp_future = stub.UnaryCall.future(_SIMPLE_REQUEST)
    _validate_status_code_and_details(
        resp_future,
        grpc.StatusCode.INTERNAL,
        "Received RST_STREAM with error code 0",
    )


def _rst_after_data(stub):
    resp_future = stub.UnaryCall.future(_SIMPLE_REQUEST)
    _validate_status_code_and_details(
        resp_future,
        grpc.StatusCode.INTERNAL,
        "Received RST_STREAM with error code 0",
    )


def _ping(stub):
    response = stub.UnaryCall(_SIMPLE_REQUEST)
    _validate_payload_type_and_length(
        response, messages_pb2.COMPRESSABLE, _RESPONSE_SIZE
    )


def _max_streams(stub):
    # send one req to ensure server sets MAX_STREAMS
    response = stub.UnaryCall(_SIMPLE_REQUEST)
    _validate_payload_type_and_length(
        response, messages_pb2.COMPRESSABLE, _RESPONSE_SIZE
    )

    # give the streams a workout
    futures = []
    for _ in range(15):
        futures.append(stub.UnaryCall.future(_SIMPLE_REQUEST))
    for future in futures:
        _validate_payload_type_and_length(
            future.result(), messages_pb2.COMPRESSABLE, _RESPONSE_SIZE
        )


def _run_test_case(test_case, stub):
    if test_case == "goaway":
        _goaway(stub)
    elif test_case == "rst_after_header":
        _rst_after_header(stub)
    elif test_case == "rst_during_data":
        _rst_during_data(stub)
    elif test_case == "rst_after_data":
        _rst_after_data(stub)
    elif test_case == "ping":
        _ping(stub)
    elif test_case == "max_streams":
        _max_streams(stub)
    else:
        raise ValueError("Invalid test case: %s" % test_case)


def _args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--server_host",
        help="the host to which to connect",
        type=str,
        default="127.0.0.1",
    )
    parser.add_argument(
        "--server_port",
        help="the port to which to connect",
        type=int,
        default="8080",
    )
    parser.add_argument(
        "--test_case",
        help="the test case to execute",
        type=str,
        default="goaway",
    )
    return parser.parse_args()


def _stub(server_host, server_port):
    target = "{}:{}".format(server_host, server_port)
    channel = grpc.insecure_channel(target)
    grpc.channel_ready_future(channel).result()
    return test_pb2_grpc.TestServiceStub(channel)


def main():
    args = _args()
    stub = _stub(args.server_host, args.server_port)
    _run_test_case(args.test_case, stub)


if __name__ == "__main__":
    main()



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import logging
from typing import AsyncIterable, Union

import grpc
from grpc.aio._metadata import Metadata
from grpc.aio._typing import MetadataKey
from grpc.aio._typing import MetadataValue
from grpc.experimental import aio

from tests.unit.framework.common import test_constants

ADHOC_METHOD = "/test/AdHoc"


def setup_absl_like_logging(level=logging.DEBUG):
    logging.basicConfig(
        level=level,
        style="{",
        format=(
            "{levelname[0]}{asctime}.{msecs:03.0f} {thread} "
            "{filename}:{lineno}] {message}"
        ),
        datefmt="%m%d %H:%M:%S",
    )


def seen_metadata(expected: Metadata, actual: Metadata):
    return not bool(set(tuple(expected)) - set(tuple(actual)))


def seen_metadatum(
    expected_key: MetadataKey, expected_value: MetadataValue, actual: Metadata
) -> bool:
    obtained = actual[expected_key]
    return obtained == expected_value


async def block_until_certain_state(
    channel: aio.Channel, expected_state: grpc.ChannelConnectivity
):
    state = channel.get_state()
    while state != expected_state:
        await channel.wait_for_state_change(state)
        state = channel.get_state()


def inject_callbacks(call: Union[aio.Call, aio.ServicerContext]):
    first_callback_ran = asyncio.Event()

    def first_callback(call):
        # Validate that all responses have been received
        # and the call is an end state.
        assert call.done()
        first_callback_ran.set()

    second_callback_ran = asyncio.Event()

    def second_callback(call):
        # Validate that all responses have been received
        # and the call is an end state.
        assert call.done()
        second_callback_ran.set()

    call.add_done_callback(first_callback)
    call.add_done_callback(second_callback)

    async def validation():
        await asyncio.wait_for(
            asyncio.gather(
                first_callback_ran.wait(), second_callback_ran.wait()
            ),
            test_constants.SHORT_TIMEOUT,
        )

    return validation()


class CountingRequestIterator:
    def __init__(self, request_iterator):
        self.request_cnt = 0
        self._request_iterator = request_iterator

    async def _forward_requests(self):
        async for request in self._request_iterator:
            self.request_cnt += 1
            yield request

    def __aiter__(self):
        return self._forward_requests()


class CountingResponseIterator:
    def __init__(self, response_iterator):
        self.response_cnt = 0
        self._response_iterator = response_iterator

    async def _forward_responses(self):
        async for response in self._response_iterator:
            self.response_cnt += 1
            yield response

    def __aiter__(self):
        return self._forward_responses()


class AdhocGenericHandler(grpc.GenericRpcHandler):
    """A generic handler to plugin testing server methods on the fly."""

    _handler: grpc.RpcMethodHandler

    def __init__(self):
        self._handler = None

    def set_adhoc_handler(self, handler: grpc.RpcMethodHandler):
        self._handler = handler

    def service(self, handler_call_details):
        if handler_call_details.method == ADHOC_METHOD:
            return self._handler
        else:
            return None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/transport_sockets/tls/v3/tls_spiffe_validator_config.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/transport_sockets/tls/v3/tls_spiffe_validator_config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nKenvoy/extensions/transport_sockets/tls/v3/tls_spiffe_validator_config.proto\x12)envoy.extensions.transport_sockets.tls.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa5\x02\n\x19SPIFFECertValidatorConfig\x12q\n\rtrust_domains\x18\x01 \x03(\x0b\x32P.envoy.extensions.transport_sockets.tls.v3.SPIFFECertValidatorConfig.TrustDomainB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x12\x37\n\rtrust_bundles\x18\x02 \x01(\x0b\x32 .envoy.config.core.v3.DataSource\x1a\\\n\x0bTrustDomain\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x36\n\x0ctrust_bundle\x18\x02 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\xba\x01\n7io.envoyproxy.envoy.extensions.transport_sockets.tls.v3B\x1dTlsSpiffeValidatorConfigProtoP\x01ZVgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3;tlsv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.transport_sockets.tls.v3.tls_spiffe_validator_config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n7io.envoyproxy.envoy.extensions.transport_sockets.tls.v3B\035TlsSpiffeValidatorConfigProtoP\001ZVgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3;tlsv3\272\200\310\321\006\002\020\002'
  _globals['_SPIFFECERTVALIDATORCONFIG_TRUSTDOMAIN'].fields_by_name['name']._loaded_options = None
  _globals['_SPIFFECERTVALIDATORCONFIG_TRUSTDOMAIN'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_SPIFFECERTVALIDATORCONFIG'].fields_by_name['trust_domains']._loaded_options = None
  _globals['_SPIFFECERTVALIDATORCONFIG'].fields_by_name['trust_domains']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_SPIFFECERTVALIDATORCONFIG']._serialized_start=212
  _globals['_SPIFFECERTVALIDATORCONFIG']._serialized_end=505
  _globals['_SPIFFECERTVALIDATORCONFIG_TRUSTDOMAIN']._serialized_start=413
  _globals['_SPIFFECERTVALIDATORCONFIG_TRUSTDOMAIN']._serialized_end=505
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/api/v2/route.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/api/v2/route.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.api.v2.core import base_pb2 as envoy_dot_api_dot_v2_dot_core_dot_base__pb2
try:
  envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2 = envoy_dot_api_dot_v2_dot_core_dot_base__pb2.envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2
except AttributeError:
  envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2 = envoy_dot_api_dot_v2_dot_core_dot_base__pb2.envoy.api.v2.core.socket_option_pb2
from envoy.api.v2.core import config_source_pb2 as envoy_dot_api_dot_v2_dot_core_dot_config__source__pb2
from envoy.api.v2.route import route_components_pb2 as envoy_dot_api_dot_v2_dot_route_dot_route__components__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x65nvoy/api/v2/route.proto\x12\x0c\x65nvoy.api.v2\x1a\x1c\x65nvoy/api/v2/core/base.proto\x1a%envoy/api/v2/core/config_source.proto\x1a)envoy/api/v2/route/route_components.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x9f\x04\n\x12RouteConfiguration\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x36\n\rvirtual_hosts\x18\x02 \x03(\x0b\x32\x1f.envoy.api.v2.route.VirtualHost\x12 \n\x04vhds\x18\t \x01(\x0b\x32\x12.envoy.api.v2.Vhds\x12/\n\x15internal_only_headers\x18\x03 \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x12P\n\x17response_headers_to_add\x18\x04 \x03(\x0b\x32$.envoy.api.v2.core.HeaderValueOptionB\t\xfa\x42\x06\x92\x01\x03\x10\xe8\x07\x12\x34\n\x1aresponse_headers_to_remove\x18\x05 \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x12O\n\x16request_headers_to_add\x18\x06 \x03(\x0b\x32$.envoy.api.v2.core.HeaderValueOptionB\t\xfa\x42\x06\x92\x01\x03\x10\xe8\x07\x12\x33\n\x19request_headers_to_remove\x18\x08 \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x12+\n#most_specific_header_mutations_wins\x18\n \x01(\x08\x12\x35\n\x11validate_clusters\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\"H\n\x04Vhds\x12@\n\rconfig_source\x18\x01 \x01(\x0b\x32\x1f.envoy.api.v2.core.ConfigSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\x8a\x01\n\x1aio.envoyproxy.envoy.api.v2B\nRouteProtoP\x01Z9github.com/envoyproxy/go-control-plane/envoy/api/v2;apiv2\xf2\x98\xfe\x8f\x05\x17\x12\x15\x65nvoy.config.route.v3\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.api.v2.route_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\032io.envoyproxy.envoy.api.v2B\nRouteProtoP\001Z9github.com/envoyproxy/go-control-plane/envoy/api/v2;apiv2\362\230\376\217\005\027\022\025envoy.config.route.v3\272\200\310\321\006\002\020\001'
  _globals['_ROUTECONFIGURATION'].fields_by_name['internal_only_headers']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['internal_only_headers']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_add']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_add']._serialized_options = b'\372B\006\222\001\003\020\350\007'
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_remove']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_remove']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_add']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_add']._serialized_options = b'\372B\006\222\001\003\020\350\007'
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_remove']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_remove']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_VHDS'].fields_by_name['config_source']._loaded_options = None
  _globals['_VHDS'].fields_by_name['config_source']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ROUTECONFIGURATION']._serialized_start=275
  _globals['_ROUTECONFIGURATION']._serialized_end=818
  _globals['_VHDS']._serialized_start=820
  _globals['_VHDS']._serialized_end=892
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import gc
import logging
import time
import unittest

import grpc
from grpc.experimental import aio

from tests.unit.framework.common import test_constants
from tests_aio.unit._test_base import AioTestBase

_UNARY_UNARY_ABORT = "/test/UnaryUnaryAbort"
_SUPPRESS_ABORT = "/test/SuppressAbort"
_REPLACE_ABORT = "/test/ReplaceAbort"
_ABORT_AFTER_REPLY = "/test/AbortAfterReply"

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x01\x01\x01"
_NUM_STREAM_RESPONSES = 5

_ABORT_CODE = grpc.StatusCode.RESOURCE_EXHAUSTED
_ABORT_DETAILS = "Phony error details"


class _GenericHandler(grpc.GenericRpcHandler):
    @staticmethod
    async def _unary_unary_abort(unused_request, context):
        await context.abort(_ABORT_CODE, _ABORT_DETAILS)
        raise RuntimeError("This line should not be executed")

    @staticmethod
    async def _suppress_abort(unused_request, context):
        try:
            await context.abort(_ABORT_CODE, _ABORT_DETAILS)
        except aio.AbortError as e:
            pass
        return _RESPONSE

    @staticmethod
    async def _replace_abort(unused_request, context):
        try:
            await context.abort(_ABORT_CODE, _ABORT_DETAILS)
        except aio.AbortError as e:
            await context.abort(
                grpc.StatusCode.INVALID_ARGUMENT, "Override abort!"
            )

    @staticmethod
    async def _abort_after_reply(unused_request, context):
        yield _RESPONSE
        await context.abort(_ABORT_CODE, _ABORT_DETAILS)
        raise RuntimeError("This line should not be executed")

    def service(self, handler_details):
        if handler_details.method == _UNARY_UNARY_ABORT:
            return grpc.unary_unary_rpc_method_handler(self._unary_unary_abort)
        if handler_details.method == _SUPPRESS_ABORT:
            return grpc.unary_unary_rpc_method_handler(self._suppress_abort)
        if handler_details.method == _REPLACE_ABORT:
            return grpc.unary_unary_rpc_method_handler(self._replace_abort)
        if handler_details.method == _ABORT_AFTER_REPLY:
            return grpc.unary_stream_rpc_method_handler(self._abort_after_reply)


async def _start_test_server():
    server = aio.server()
    port = server.add_insecure_port("[::]:0")
    server.add_generic_rpc_handlers((_GenericHandler(),))
    await server.start()
    return "localhost:%d" % port, server


class TestAbort(AioTestBase):
    async def setUp(self):
        address, self._server = await _start_test_server()
        self._channel = aio.insecure_channel(address)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def test_unary_unary_abort(self):
        method = self._channel.unary_unary(_UNARY_UNARY_ABORT)
        call = method(_REQUEST)

        self.assertEqual(_ABORT_CODE, await call.code())
        self.assertEqual(_ABORT_DETAILS, await call.details())

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call

        rpc_error = exception_context.exception
        self.assertEqual(_ABORT_CODE, rpc_error.code())
        self.assertEqual(_ABORT_DETAILS, rpc_error.details())

    async def test_suppress_abort(self):
        method = self._channel.unary_unary(_SUPPRESS_ABORT)
        call = method(_REQUEST)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call

        rpc_error = exception_context.exception
        self.assertEqual(_ABORT_CODE, rpc_error.code())
        self.assertEqual(_ABORT_DETAILS, rpc_error.details())

    async def test_replace_abort(self):
        method = self._channel.unary_unary(_REPLACE_ABORT)
        call = method(_REQUEST)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call

        rpc_error = exception_context.exception
        self.assertEqual(_ABORT_CODE, rpc_error.code())
        self.assertEqual(_ABORT_DETAILS, rpc_error.details())

    async def test_abort_after_reply(self):
        method = self._channel.unary_stream(_ABORT_AFTER_REPLY)
        call = method(_REQUEST)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.read()
            await call.read()

        rpc_error = exception_context.exception
        self.assertEqual(_ABORT_CODE, rpc_error.code())
        self.assertEqual(_ABORT_DETAILS, rpc_error.details())

        self.assertEqual(_ABORT_CODE, await call.code())
        self.assertEqual(_ABORT_DETAILS, await call.details())


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



#!/usr/bin/python
# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This script analyzes link map file generated by Xcode. It calculates and
# prints out the sizes of each dependent library and the total sizes of the
# symbols.
# The script takes one parameter, which is the path to the link map file.

import re
import sys


def parse_link_map(filename):
    table_tag = {}
    state = "start"

    table_stats_symbol = {}
    table_stats_dead = {}
    section_total_size = 0
    symbol_total_size = 0

    boringssl_size = 0
    core_size = 0
    objc_size = 0
    protobuf_size = 0

    lines = open(filename, encoding="utf-8", errors="ignore").readlines()
    for line in lines:
        line_stripped = line[:-1]
        if "# Object files:" == line_stripped:
            state = "object"
            continue
        elif "# Sections:" == line_stripped:
            state = "section"
            continue
        elif "# Symbols:" == line_stripped:
            state = "symbol"
            continue
        elif "# Dead Stripped Symbols:" == line_stripped:
            state = "dead"
            continue

        if state == "object":
            segs = re.search("(\[ *[0-9]*\]) (.*)", line_stripped)
            table_tag[segs.group(1)] = segs.group(2)

        if state == "section":
            if len(line_stripped) == 0 or line_stripped[0] == "#":
                continue
            segs = re.search("^(.+?)\s+(.+?)\s+.*", line_stripped)
            section_total_size += int(segs.group(2), 16)

        if state == "symbol":
            if len(line_stripped) == 0 or line_stripped[0] == "#":
                continue
            segs = re.search("^.+?\s+(.+?)\s+(\[.+?\]).*", line_stripped)
            if not segs:
                continue
            target = table_tag[segs.group(2)]
            target_stripped = re.search("^(.*?)(\(.+?\))?$", target).group(1)
            size = int(segs.group(1), 16)
            if not target_stripped in table_stats_symbol:
                table_stats_symbol[target_stripped] = 0
            table_stats_symbol[target_stripped] += size
            if "BoringSSL" in target_stripped:
                boringssl_size += size
            elif "libgRPC-Core" in target_stripped:
                core_size += size
            elif (
                "libgRPC-RxLibrary" in target_stripped
                or "libgRPC" in target_stripped
                or "libgRPC-ProtoLibrary" in target_stripped
            ):
                objc_size += size
            elif "libProtobuf" in target_stripped:
                protobuf_size += size

    for target in table_stats_symbol:
        symbol_total_size += table_stats_symbol[target]

    return (
        core_size,
        objc_size,
        boringssl_size,
        protobuf_size,
        symbol_total_size,
    )


def main():
    filename = sys.argv[1]
    (
        core_size,
        objc_size,
        boringssl_size,
        protobuf_size,
        total_size,
    ) = parse_link_map(filename)
    print("Core size:{:,}".format(core_size))
    print("ObjC size:{:,}".format(objc_size))
    print("BoringSSL size:{:,}".format(boringssl_size))
    print("Protobuf size:{:,}\n".format(protobuf_size))
    print("Total size:{:,}".format(total_size))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import fnmatch
import os
import re
import sys

import yaml

_RE_API = r"(?:GPRAPI|GRPCAPI|CENSUSAPI)([^#;]*);"


def list_c_apis(filenames):
    for filename in filenames:
        with open(filename, "r") as f:
            text = f.read()
        for m in re.finditer(_RE_API, text):
            api_declaration = re.sub("[ \r\n\t]+", " ", m.group(1))
            type_and_name, args_and_close = api_declaration.split("(", 1)
            args = args_and_close[: args_and_close.rfind(")")].strip()
            last_space = type_and_name.rfind(" ")
            last_star = type_and_name.rfind("*")
            type_end = max(last_space, last_star)
            return_type = type_and_name[0 : type_end + 1].strip()
            name = type_and_name[type_end + 1 :].strip()
            yield {
                "return_type": return_type,
                "name": name,
                "arguments": args,
                "header": filename,
            }


def headers_under(directory):
    for root, dirnames, filenames in os.walk(directory):
        for filename in fnmatch.filter(filenames, "*.h"):
            yield os.path.join(root, filename)


def mako_plugin(dictionary):
    apis = []
    headers = []

    for lib in dictionary["libs"]:
        if lib["name"] in ["grpc", "gpr"]:
            headers.extend(lib["public_headers"])

    apis.extend(list_c_apis(sorted(set(headers))))
    dictionary["c_apis"] = apis


if __name__ == "__main__":
    print(
        (yaml.dump([api for api in list_c_apis(headers_under("include/grpc"))]))
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/ratelimit/v2/rls.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/ratelimit/v2/rls.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.api.v2.core import grpc_service_pb2 as envoy_dot_api_dot_v2_dot_core_dot_grpc__service__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#envoy/config/ratelimit/v2/rls.proto\x12\x19\x65nvoy.config.ratelimit.v2\x1a$envoy/api/v2/core/grpc_service.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"d\n\x16RateLimitServiceConfig\x12>\n\x0cgrpc_service\x18\x02 \x01(\x0b\x32\x1e.envoy.api.v2.core.GrpcServiceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01J\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04\x42\x8b\x01\n\'io.envoyproxy.envoy.config.ratelimit.v2B\x08RlsProtoP\x01ZLgithub.com/envoyproxy/go-control-plane/envoy/config/ratelimit/v2;ratelimitv2\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.ratelimit.v2.rls_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\'io.envoyproxy.envoy.config.ratelimit.v2B\010RlsProtoP\001ZLgithub.com/envoyproxy/go-control-plane/envoy/config/ratelimit/v2;ratelimitv2\272\200\310\321\006\002\020\001'
  _globals['_RATELIMITSERVICECONFIG'].fields_by_name['grpc_service']._loaded_options = None
  _globals['_RATELIMITSERVICECONFIG'].fields_by_name['grpc_service']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_RATELIMITSERVICECONFIG']._serialized_start=160
  _globals['_RATELIMITSERVICECONFIG']._serialized_end=260
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from src.proto.grpc.testing import benchmark_service_pb2_grpc
from src.proto.grpc.testing import messages_pb2


class BenchmarkServer(benchmark_service_pb2_grpc.BenchmarkServiceServicer):
    """Synchronous Server implementation for the Benchmark service."""

    def UnaryCall(self, request, context):
        payload = messages_pb2.Payload(body=b"\0" * request.response_size)
        return messages_pb2.SimpleResponse(payload=payload)

    def StreamingCall(self, request_iterator, context):
        for request in request_iterator:
            payload = messages_pb2.Payload(body=b"\0" * request.response_size)
            yield messages_pb2.SimpleResponse(payload=payload)


class GenericBenchmarkServer(
    benchmark_service_pb2_grpc.BenchmarkServiceServicer
):
    """Generic Server implementation for the Benchmark service."""

    def __init__(self, resp_size):
        self._response = b"\0" * resp_size

    def UnaryCall(self, request, context):
        return self._response

    def StreamingCall(self, request_iterator, context):
        for request in request_iterator:
            yield self._response



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/api/http.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/api/http.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15google/api/http.proto\x12\ngoogle.api\"T\n\x04Http\x12#\n\x05rules\x18\x01 \x03(\x0b\x32\x14.google.api.HttpRule\x12\'\n\x1f\x66ully_decode_reserved_expansion\x18\x02 \x01(\x08\"\x81\x02\n\x08HttpRule\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\r\n\x03get\x18\x02 \x01(\tH\x00\x12\r\n\x03put\x18\x03 \x01(\tH\x00\x12\x0e\n\x04post\x18\x04 \x01(\tH\x00\x12\x10\n\x06\x64\x65lete\x18\x05 \x01(\tH\x00\x12\x0f\n\x05patch\x18\x06 \x01(\tH\x00\x12/\n\x06\x63ustom\x18\x08 \x01(\x0b\x32\x1d.google.api.CustomHttpPatternH\x00\x12\x0c\n\x04\x62ody\x18\x07 \x01(\t\x12\x15\n\rresponse_body\x18\x0c \x01(\t\x12\x31\n\x13\x61\x64\x64itional_bindings\x18\x0b \x03(\x0b\x32\x14.google.api.HttpRuleB\t\n\x07pattern\"/\n\x11\x43ustomHttpPattern\x12\x0c\n\x04kind\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\tBg\n\x0e\x63om.google.apiB\tHttpProtoP\x01ZAgoogle.golang.org/genproto/googleapis/api/annotations;annotations\xa2\x02\x04GAPIb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.api.http_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\016com.google.apiB\tHttpProtoP\001ZAgoogle.golang.org/genproto/googleapis/api/annotations;annotations\242\002\004GAPI'
  _globals['_HTTP']._serialized_start=37
  _globals['_HTTP']._serialized_end=121
  _globals['_HTTPRULE']._serialized_start=124
  _globals['_HTTPRULE']._serialized_end=381
  _globals['_CUSTOMHTTPPATTERN']._serialized_start=383
  _globals['_CUSTOMHTTPPATTERN']._serialized_end=430
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket

cdef gpr_timespec _GPR_INF_FUTURE = gpr_inf_future(GPR_CLOCK_REALTIME)
cdef float _POLL_AWAKE_INTERVAL_S = 0.2

# This bool indicates if the event loop impl can monitor a given fd, or has
# loop.add_reader method.
cdef bint _has_fd_monitoring = True

cdef void _unified_socket_write(int fd) noexcept nogil:
    _unified_socket_write_impl(fd)


def _handle_callback_wrapper(CallbackWrapper callback_wrapper, int success):
    CallbackWrapper.functor_run(callback_wrapper.c_functor(), success)


cdef class BaseCompletionQueue:

    cdef grpc_completion_queue* c_ptr(self):
        return self._cq


cdef class _BoundEventLoop:

    def __cinit__(self, object loop, object read_socket, object handler):
        global _has_fd_monitoring
        self.loop = loop
        self.read_socket = read_socket
        reader_function = functools.partial(
            handler,
            loop
        )
        # NOTE(lidiz) There isn't a way to cleanly pre-check if fd monitoring
        # support is available or not. Checking the event loop policy is not
        # good enough. The application can have its own loop implementation, or
        # uses different types of event loops (e.g., 1 Proactor, 3 Selectors).
        if _has_fd_monitoring:
            try:
                self.loop.add_reader(self.read_socket, reader_function)
                self._has_reader = True
            except NotImplementedError:
                _has_fd_monitoring = False
                self._has_reader = False

    def close(self):
        if self.loop:
            if self._has_reader:
                self.loop.remove_reader(self.read_socket)


cdef class PollerCompletionQueue(BaseCompletionQueue):

    def __cinit__(self):
        self._cq = grpc_completion_queue_create_for_next(NULL)
        self._shutdown = False
        self._poller_thread = threading.Thread(target=self._poll_wrapper, daemon=True)
        self._poller_thread.start()

        self._read_socket, self._write_socket = socket.socketpair()
        self._write_fd = self._write_socket.fileno()
        self._loops = {}

        # The read socket might be read by multiple threads. But only one of them will
        # read the 1 byte sent by the poller thread. This setting is essential to allow
        # multiple loops in multiple threads bound to the same poller.
        self._read_socket.setblocking(False)

        self._queue = cpp_event_queue()

    def bind_loop(self, object loop):
        if loop in self._loops:
            return
        else:
            self._loops[loop] = _BoundEventLoop(loop, self._read_socket, self._handle_events)

    cdef int _poll(self) except -1 nogil:
        cdef grpc_event event
        cdef CallbackContext *context

        while not self._shutdown:
            event = grpc_completion_queue_next(self._cq,
                                               _GPR_INF_FUTURE,
                                               NULL)

            if event.type == GRPC_QUEUE_TIMEOUT:
                with gil:
                    raise AssertionError("Core should not return GRPC_QUEUE_TIMEOUT!")
            elif event.type == GRPC_QUEUE_SHUTDOWN:
                self._shutdown = True
            else:
                self._queue_mutex.lock()
                self._queue.push(event)
                self._queue_mutex.unlock()
                if _has_fd_monitoring:
                    _unified_socket_write(self._write_fd)
                else:
                    with gil:
                        # Event loops can be paused or killed at any time. So,
                        # instead of delegate to any thread, the polling thread
                        # should handle the distribution of the event.
                        self._handle_events(None)
        return 0

    def _poll_wrapper(self):
        with nogil:
            self._poll()

    cdef shutdown(self):
        # Removes the socket hook from loops
        for loop in self._loops:
            self._loops.get(loop).close()

        # Close the read socket to prevent the `_poller_thread` from blocking on a `write` syscall
        # when the Unix-domain socket buffer is full. Once the loops above are closed, the read
        # socket is no longer being read, so close it to avoid `write` syscall hangs.
        #
        # See `sock_alloc_send_pskb` for more details about these `write` syscall hangs.
        self._read_socket.close()

        # TODO(https://github.com/grpc/grpc/issues/22365) perform graceful shutdown
        grpc_completion_queue_shutdown(self._cq)
        while not self._shutdown:
            self._poller_thread.join(timeout=_POLL_AWAKE_INTERVAL_S)
        grpc_completion_queue_destroy(self._cq)

        # Clean up the write socket
        self._write_socket.close()

    def _handle_events(self, object context_loop):
        cdef bytes data
        if _has_fd_monitoring:
            # If fd monitoring is working, clean the socket without blocking.
            try:
                # In case of multiple loops, the read socket might be read by multiple threads.
                # But only one of them will read the 1 byte sent by the poller thread.
                # So, we need to handle the case where the socket is already empty.
                data = self._read_socket.recv(1)
            except BlockingIOError:
                pass
        cdef grpc_event event
        cdef CallbackContext *context

        while True:
            self._queue_mutex.lock()
            if self._queue.empty():
                self._queue_mutex.unlock()
                break
            else:
                event = self._queue.front()
                self._queue.pop()
                self._queue_mutex.unlock()

            context = <CallbackContext *>event.tag
            loop = <object>context.loop
            if loop is context_loop:
                # Executes callbacks: complete the future
                CallbackWrapper.functor_run(
                    <grpc_completion_queue_functor *>event.tag,
                    event.success
                )
            else:
                loop.call_soon_threadsafe(
                    _handle_callback_wrapper,
                    <CallbackWrapper>context.callback_wrapper,
                    event.success
                )



# Copyright 2024 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Sequence, Optional

import unittest
import sys
import pkgutil
import importlib.util

class SingleLoader:
    def __init__(self, pattern: str, unittest_path: str):
        loader = unittest.TestLoader()
        self.suite = unittest.TestSuite()
        tests = []

        for importer, module_name, is_package in pkgutil.walk_packages([unittest_path]):
            if pattern in module_name:
                try:
                    spec = importer.find_spec(module_name)
                    if spec is not None:
                        module = importlib.util.module_from_spec(spec)
                        spec.loader.exec_module(module)
                        tests.append(loader.loadTestsFromModule(module))
                except Exception as e:
                  raise AssertionError(f"Error loading module {module_name}: {e}")

        if len(tests) != 1:            
            raise AssertionError("Expected only 1 test module. Found {}".format(tests))
        self.suite.addTest(tests[0])

    def loadTestsFromNames(self, names: Sequence[str], module: Optional[str] = None) -> unittest.TestSuite:
        return self.suite

if __name__ == "__main__":

    if len(sys.argv) != 3:
        print(f"USAGE: {sys.argv[0]} TARGET_MODULE", file=sys.stderr)
        sys.exit(1)


    target_module = sys.argv[1]
    unittest_path = sys.argv[2]

    loader = SingleLoader(target_module, unittest_path)
    runner = unittest.TextTestRunner(verbosity=0)
    result = runner.run(loader.suite)
    
    if not result.wasSuccessful():
        sys.exit("Test failure.")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: opencensus/proto/trace/v1/trace.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'opencensus/proto/trace/v1/trace.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from opencensus.proto.resource.v1 import resource_pb2 as opencensus_dot_proto_dot_resource_dot_v1_dot_resource__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%opencensus/proto/trace/v1/trace.proto\x12\x19opencensus.proto.trace.v1\x1a+opencensus/proto/resource/v1/resource.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\"\xac\x12\n\x04Span\x12\x10\n\x08trace_id\x18\x01 \x01(\x0c\x12\x0f\n\x07span_id\x18\x02 \x01(\x0c\x12>\n\ntracestate\x18\x0f \x01(\x0b\x32*.opencensus.proto.trace.v1.Span.Tracestate\x12\x16\n\x0eparent_span_id\x18\x03 \x01(\x0c\x12:\n\x04name\x18\x04 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\x12\x36\n\x04kind\x18\x0e \x01(\x0e\x32(.opencensus.proto.trace.v1.Span.SpanKind\x12.\n\nstart_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12>\n\nattributes\x18\x07 \x01(\x0b\x32*.opencensus.proto.trace.v1.Span.Attributes\x12:\n\x0bstack_trace\x18\x08 \x01(\x0b\x32%.opencensus.proto.trace.v1.StackTrace\x12?\n\x0btime_events\x18\t \x01(\x0b\x32*.opencensus.proto.trace.v1.Span.TimeEvents\x12\x34\n\x05links\x18\n \x01(\x0b\x32%.opencensus.proto.trace.v1.Span.Links\x12\x31\n\x06status\x18\x0b \x01(\x0b\x32!.opencensus.proto.trace.v1.Status\x12\x38\n\x08resource\x18\x10 \x01(\x0b\x32&.opencensus.proto.resource.v1.Resource\x12?\n\x1bsame_process_as_parent_span\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x36\n\x10\x63hild_span_count\x18\r \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x1at\n\nTracestate\x12\x41\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x30.opencensus.proto.trace.v1.Span.Tracestate.Entry\x1a#\n\x05\x45ntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x1a\xe3\x01\n\nAttributes\x12S\n\rattribute_map\x18\x01 \x03(\x0b\x32<.opencensus.proto.trace.v1.Span.Attributes.AttributeMapEntry\x12 \n\x18\x64ropped_attributes_count\x18\x02 \x01(\x05\x1a^\n\x11\x41ttributeMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x38\n\x05value\x18\x02 \x01(\x0b\x32).opencensus.proto.trace.v1.AttributeValue:\x02\x38\x01\x1a\xbf\x04\n\tTimeEvent\x12(\n\x04time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12J\n\nannotation\x18\x02 \x01(\x0b\x32\x34.opencensus.proto.trace.v1.Span.TimeEvent.AnnotationH\x00\x12O\n\rmessage_event\x18\x03 \x01(\x0b\x32\x36.opencensus.proto.trace.v1.Span.TimeEvent.MessageEventH\x00\x1a\x8f\x01\n\nAnnotation\x12\x41\n\x0b\x64\x65scription\x18\x01 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\x12>\n\nattributes\x18\x02 \x01(\x0b\x32*.opencensus.proto.trace.v1.Span.Attributes\x1a\xcf\x01\n\x0cMessageEvent\x12I\n\x04type\x18\x01 \x01(\x0e\x32;.opencensus.proto.trace.v1.Span.TimeEvent.MessageEvent.Type\x12\n\n\x02id\x18\x02 \x01(\x04\x12\x19\n\x11uncompressed_size\x18\x03 \x01(\x04\x12\x17\n\x0f\x63ompressed_size\x18\x04 \x01(\x04\"4\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x08\n\x04SENT\x10\x01\x12\x0c\n\x08RECEIVED\x10\x02\x42\x07\n\x05value\x1a\x94\x01\n\nTimeEvents\x12=\n\ntime_event\x18\x01 \x03(\x0b\x32).opencensus.proto.trace.v1.Span.TimeEvent\x12!\n\x19\x64ropped_annotations_count\x18\x02 \x01(\x05\x12$\n\x1c\x64ropped_message_events_count\x18\x03 \x01(\x05\x1a\xaf\x02\n\x04Link\x12\x10\n\x08trace_id\x18\x01 \x01(\x0c\x12\x0f\n\x07span_id\x18\x02 \x01(\x0c\x12\x37\n\x04type\x18\x03 \x01(\x0e\x32).opencensus.proto.trace.v1.Span.Link.Type\x12>\n\nattributes\x18\x04 \x01(\x0b\x32*.opencensus.proto.trace.v1.Span.Attributes\x12>\n\ntracestate\x18\x05 \x01(\x0b\x32*.opencensus.proto.trace.v1.Span.Tracestate\"K\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11\x43HILD_LINKED_SPAN\x10\x01\x12\x16\n\x12PARENT_LINKED_SPAN\x10\x02\x1aX\n\x05Links\x12\x32\n\x04link\x18\x01 \x03(\x0b\x32$.opencensus.proto.trace.v1.Span.Link\x12\x1b\n\x13\x64ropped_links_count\x18\x02 \x01(\x05\"=\n\x08SpanKind\x12\x19\n\x15SPAN_KIND_UNSPECIFIED\x10\x00\x12\n\n\x06SERVER\x10\x01\x12\n\n\x06\x43LIENT\x10\x02\"\'\n\x06Status\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xa2\x01\n\x0e\x41ttributeValue\x12\x44\n\x0cstring_value\x18\x01 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableStringH\x00\x12\x13\n\tint_value\x18\x02 \x01(\x03H\x00\x12\x14\n\nbool_value\x18\x03 \x01(\x08H\x00\x12\x16\n\x0c\x64ouble_value\x18\x04 \x01(\x01H\x00\x42\x07\n\x05value\"\xed\x04\n\nStackTrace\x12G\n\x0cstack_frames\x18\x01 \x01(\x0b\x32\x31.opencensus.proto.trace.v1.StackTrace.StackFrames\x12\x1b\n\x13stack_trace_hash_id\x18\x02 \x01(\x04\x1a\x8a\x03\n\nStackFrame\x12\x43\n\rfunction_name\x18\x01 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\x12L\n\x16original_function_name\x18\x02 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\x12?\n\tfile_name\x18\x03 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\x12\x13\n\x0bline_number\x18\x04 \x01(\x03\x12\x15\n\rcolumn_number\x18\x05 \x01(\x03\x12\x36\n\x0bload_module\x18\x06 \x01(\x0b\x32!.opencensus.proto.trace.v1.Module\x12\x44\n\x0esource_version\x18\x07 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\x1al\n\x0bStackFrames\x12?\n\x05\x66rame\x18\x01 \x03(\x0b\x32\x30.opencensus.proto.trace.v1.StackTrace.StackFrame\x12\x1c\n\x14\x64ropped_frames_count\x18\x02 \x01(\x05\"\x86\x01\n\x06Module\x12<\n\x06module\x18\x01 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\x12>\n\x08\x62uild_id\x18\x02 \x01(\x0b\x32,.opencensus.proto.trace.v1.TruncatableString\"@\n\x11TruncatableString\x12\r\n\x05value\x18\x01 \x01(\t\x12\x1c\n\x14truncated_byte_count\x18\x02 \x01(\x05\x42\x8c\x01\n\x1cio.opencensus.proto.trace.v1B\nTraceProtoP\x01ZBgithub.com/census-instrumentation/opencensus-proto/gen-go/trace/v1\xea\x02\x19OpenCensus.Proto.Trace.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'opencensus.proto.trace.v1.trace_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034io.opencensus.proto.trace.v1B\nTraceProtoP\001ZBgithub.com/census-instrumentation/opencensus-proto/gen-go/trace/v1\352\002\031OpenCensus.Proto.Trace.V1'
  _globals['_SPAN_ATTRIBUTES_ATTRIBUTEMAPENTRY']._loaded_options = None
  _globals['_SPAN_ATTRIBUTES_ATTRIBUTEMAPENTRY']._serialized_options = b'8\001'
  _globals['_SPAN']._serialized_start=179
  _globals['_SPAN']._serialized_end=2527
  _globals['_SPAN_TRACESTATE']._serialized_start=993
  _globals['_SPAN_TRACESTATE']._serialized_end=1109
  _globals['_SPAN_TRACESTATE_ENTRY']._serialized_start=1074
  _globals['_SPAN_TRACESTATE_ENTRY']._serialized_end=1109
  _globals['_SPAN_ATTRIBUTES']._serialized_start=1112
  _globals['_SPAN_ATTRIBUTES']._serialized_end=1339
  _globals['_SPAN_ATTRIBUTES_ATTRIBUTEMAPENTRY']._serialized_start=1245
  _globals['_SPAN_ATTRIBUTES_ATTRIBUTEMAPENTRY']._serialized_end=1339
  _globals['_SPAN_TIMEEVENT']._serialized_start=1342
  _globals['_SPAN_TIMEEVENT']._serialized_end=1917
  _globals['_SPAN_TIMEEVENT_ANNOTATION']._serialized_start=1555
  _globals['_SPAN_TIMEEVENT_ANNOTATION']._serialized_end=1698
  _globals['_SPAN_TIMEEVENT_MESSAGEEVENT']._serialized_start=1701
  _globals['_SPAN_TIMEEVENT_MESSAGEEVENT']._serialized_end=1908
  _globals['_SPAN_TIMEEVENT_MESSAGEEVENT_TYPE']._serialized_start=1856
  _globals['_SPAN_TIMEEVENT_MESSAGEEVENT_TYPE']._serialized_end=1908
  _globals['_SPAN_TIMEEVENTS']._serialized_start=1920
  _globals['_SPAN_TIMEEVENTS']._serialized_end=2068
  _globals['_SPAN_LINK']._serialized_start=2071
  _globals['_SPAN_LINK']._serialized_end=2374
  _globals['_SPAN_LINK_TYPE']._serialized_start=2299
  _globals['_SPAN_LINK_TYPE']._serialized_end=2374
  _globals['_SPAN_LINKS']._serialized_start=2376
  _globals['_SPAN_LINKS']._serialized_end=2464
  _globals['_SPAN_SPANKIND']._serialized_start=2466
  _globals['_SPAN_SPANKIND']._serialized_end=2527
  _globals['_STATUS']._serialized_start=2529
  _globals['_STATUS']._serialized_end=2568
  _globals['_ATTRIBUTEVALUE']._serialized_start=2571
  _globals['_ATTRIBUTEVALUE']._serialized_end=2733
  _globals['_STACKTRACE']._serialized_start=2736
  _globals['_STACKTRACE']._serialized_end=3357
  _globals['_STACKTRACE_STACKFRAME']._serialized_start=2853
  _globals['_STACKTRACE_STACKFRAME']._serialized_end=3247
  _globals['_STACKTRACE_STACKFRAMES']._serialized_start=3249
  _globals['_STACKTRACE_STACKFRAMES']._serialized_end=3357
  _globals['_MODULE']._serialized_start=3360
  _globals['_MODULE']._serialized_end=3494
  _globals['_TRUNCATABLESTRING']._serialized_start=3496
  _globals['_TRUNCATABLESTRING']._serialized_end=3560
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import binascii
import collections
import ctypes
import datetime
import json
import math
import os
import re
import sys

import yaml

with open("src/core/config/config_vars.yaml") as f:
    attrs = yaml.safe_load(f.read())

error = False
today = datetime.date.today()
two_quarters_from_now = today + datetime.timedelta(days=180)
for attr in attrs:
    if "name" not in attr:
        print("config has no name: %r" % attr)
        error = True
        continue
    if (
        "experiment" in attr["name"] and "experimental" not in attr["name"]
    ) and attr["name"] != "experiments":
        print("use experiment system for experiments")
        error = True
    if "description" not in attr:
        print("no description for %s" % attr["name"])
        error = True
    if "default" not in attr:
        print("no default for %s" % attr["name"])
        error = True

if error:
    sys.exit(1)


def c_str(s, encoding="ascii"):
    if s is None:
        return '""'
    if isinstance(s, str):
        s = s.encode(encoding)
    result = ""
    for c in s:
        c = chr(c) if isinstance(c, int) else c
        if not (32 <= ord(c) < 127) or c in ("\\", '"'):
            result += "\\%03o" % ord(c)
        else:
            result += c
    return '"' + result + '"'


def snake_to_pascal(s):
    return "".join(x.capitalize() for x in s.split("_"))


# utility: print a big comment block into a set of files
def put_banner(files, banner):
    for f in files:
        for line in banner:
            print("// %s" % line, file=f)
        print(file=f)


def put_copyright(file):
    # copy-paste copyright notice from this file
    with open(sys.argv[0]) as my_source:
        copyright = []
        for line in my_source:
            if line[0] != "#":
                break
        for line in my_source:
            if line[0] == "#":
                copyright.append(line)
                break
        for line in my_source:
            if line[0] != "#":
                break
            copyright.append(line)
        put_banner([file], [line[2:].rstrip() for line in copyright])


RETURN_TYPE = {
    "int": "int32_t",
    "double": "double",
    "string": "absl::string_view",
    "comma_separated_string": "absl::string_view",
    "bool": "bool",
}

MEMBER_TYPE = {
    "int": "int32_t",
    "double": "double",
    "string": "std::string",
    "comma_separated_string": "std::string",
    "bool": "bool",
}

FLAG_TYPE = {
    "int": "absl::optional<int32_t>",
    "double": "absl::optional<double>",
    "string": "absl::optional<std::string>",
    "comma_separated_string": "std::vector<std::string>",
    "bool": "absl::optional<bool>",
}

PROTO_TYPE = {
    "int": "int32",
    "double": "double",
    "string": "string",
    "comma_separated_string": "string",
    "bool": "bool",
}

SORT_ORDER_FOR_PACKING = {
    "int": 0,
    "double": 1,
    "bool": 2,
    "string": 3,
    "comma_separated_string": 4,
}


def bool_default_value(x, name):
    if x == True:
        return "true"
    if x == False:
        return "false"
    if isinstance(x, str) and x.startswith("$"):
        return x[1:]
    return x


def int_default_value(x, name):
    if isinstance(x, str) and x.startswith("$"):
        return x[1:]
    return x


def double_default_value(x, name):
    if isinstance(x, str) and x.startswith("$"):
        return x[1:]
    return x


def string_default_value(x, name):
    if x is None:
        return '""'
    if x.startswith("$"):
        return x[1:]
    return c_str(x)


DEFAULT_VALUE = {
    "int": int_default_value,
    "double": double_default_value,
    "bool": bool_default_value,
    "string": string_default_value,
    "comma_separated_string": string_default_value,
}

TO_STRING = {
    "int": "$",
    "double": "$",
    "bool": '$?"true":"false"',
    "string": '"\\"", absl::CEscape($), "\\""',
    "comma_separated_string": '"\\"", absl::CEscape($), "\\""',
}

attrs_in_packing_order = sorted(
    attrs, key=lambda a: SORT_ORDER_FOR_PACKING[a["type"]]
)

with open("test/core/test_util/fuzz_config_vars.proto", "w") as P:
    put_copyright(P)

    put_banner(
        [P],
        [
            "",
            "Automatically generated by tools/codegen/core/gen_config_vars.py",
            "",
        ],
    )

    print('syntax = "proto3";', file=P)
    print(file=P)
    print("package grpc.testing;", file=P)
    print(file=P)
    print("message FuzzConfigVars {", file=P)
    for attr in attrs_in_packing_order:
        if attr.get("fuzz", False) == False:
            continue
        print(
            "  optional %s %s = %d;"
            % (
                PROTO_TYPE[attr["type"]],
                attr["name"],
                binascii.crc32(attr["name"].encode("ascii")) & 0x1FFFFFFF,
            ),
            file=P,
        )
    print("};", file=P)

with open("test/core/test_util/fuzz_config_vars.cc", "w") as C:
    put_copyright(C)

    put_banner(
        [C],
        [
            "",
            "Automatically generated by tools/codegen/core/gen_config_vars.py",
            "",
        ],
    )

    print('#include "test/core/test_util/fuzz_config_vars.h"', file=C)
    print(file=C)
    print("namespace grpc_core {", file=C)
    print(file=C)
    print(
        (
            "ConfigVars::Overrides OverridesFromFuzzConfigVars(const"
            " grpc::testing::FuzzConfigVars& vars) {"
        ),
        file=C,
    )
    print("  ConfigVars::Overrides overrides;", file=C)
    for attr in attrs_in_packing_order:
        fuzz = attr.get("fuzz", False)
        if not fuzz:
            continue
        assert fuzz == True
        print("  if (vars.has_%s()) {" % attr["name"], file=C)
        print(
            "    overrides.%s = vars.%s();" % (attr["name"], attr["name"]),
            file=C,
        )
        print("  }", file=C)
    print("  return overrides;", file=C)
    print("}", file=C)
    print(
        "void ApplyFuzzConfigVars(const grpc::testing::FuzzConfigVars& vars) {",
        file=C,
    )
    print(
        "  ConfigVars::SetOverrides(OverridesFromFuzzConfigVars(vars));", file=C
    )
    print("}", file=C)
    print(file=C)
    print("}  // namespace grpc_core", file=C)

with open("src/core/config/config_vars.h", "w") as H:
    put_copyright(H)

    put_banner(
        [H],
        [
            "",
            "Automatically generated by tools/codegen/core/gen_config_vars.py",
            "",
        ],
    )

    print("#ifndef GRPC_SRC_CORE_CONFIG_CONFIG_VARS_H", file=H)
    print("#define GRPC_SRC_CORE_CONFIG_CONFIG_VARS_H", file=H)
    print(file=H)
    print("#include <grpc/support/port_platform.h>", file=H)
    print(file=H)
    print("#include <string>", file=H)
    print("#include <atomic>", file=H)
    print("#include <optional>", file=H)
    print("#include <stdint.h>", file=H)
    print('#include "absl/types/optional.h"', file=H)
    print('#include "absl/strings/string_view.h"', file=H)
    print(file=H)
    print("namespace grpc_core {", file=H)
    print(file=H)
    print("class GPR_DLL ConfigVars {", file=H)
    print(" public:", file=H)
    print("  struct Overrides {", file=H)
    for attr in attrs_in_packing_order:
        print(
            "    absl::optional<%s> %s;"
            % (MEMBER_TYPE[attr["type"]], attr["name"]),
            file=H,
        )
    print("  };", file=H)
    print("  ConfigVars(const ConfigVars&) = delete;", file=H)
    print("  ConfigVars& operator=(const ConfigVars&) = delete;", file=H)
    print(
        "  // Get the core configuration; if it does not exist, create it.",
        file=H,
    )
    print("  static const ConfigVars& Get() {", file=H)
    print("    auto* p = config_vars_.load(std::memory_order_acquire);", file=H)
    print("    if (p != nullptr) return *p;", file=H)
    print("    return Load();", file=H)
    print("  }", file=H)
    print("  static void SetOverrides(const Overrides& overrides);", file=H)
    print(
        "  // Drop the config vars. Users must ensure no other threads are",
        file=H,
    )
    print("  // accessing the configuration.", file=H)
    print("  static void Reset();", file=H)
    print("  std::string ToString() const;", file=H)
    for attr in attrs:
        for line in attr["description"].splitlines():
            print("  // %s" % line, file=H)
        if attr.get("force-load-on-access", False):
            print(
                "  %s %s() const;"
                % (MEMBER_TYPE[attr["type"]], snake_to_pascal(attr["name"])),
                file=H,
            )
        else:
            print(
                "  %s %s() const { return %s_; }"
                % (
                    RETURN_TYPE[attr["type"]],
                    snake_to_pascal(attr["name"]),
                    attr["name"],
                ),
                file=H,
            )
    print(" private:", file=H)
    print("  explicit ConfigVars(const Overrides& overrides);", file=H)
    print("  static const ConfigVars& Load();", file=H)
    print("  static std::atomic<ConfigVars*> config_vars_;", file=H)
    for attr in attrs_in_packing_order:
        if attr.get("force-load-on-access", False):
            continue
        print("  %s %s_;" % (MEMBER_TYPE[attr["type"]], attr["name"]), file=H)
    for attr in attrs_in_packing_order:
        if attr.get("force-load-on-access", False) == False:
            continue
        print(
            "  absl::optional<%s> override_%s_;"
            % (MEMBER_TYPE[attr["type"]], attr["name"]),
            file=H,
        )
    print("};", file=H)
    print(file=H)
    print("}  // namespace grpc_core", file=H)
    print(file=H)
    print("#endif  // GRPC_SRC_CORE_CONFIG_CONFIG_VARS_H", file=H)

with open("src/core/config/config_vars.cc", "w") as C:
    put_copyright(C)

    put_banner(
        [C],
        [
            "",
            "Automatically generated by tools/codegen/core/gen_config_vars.py",
            "",
        ],
    )

    print("#include <grpc/support/port_platform.h>", file=C)
    print('#include "src/core/config/config_vars.h"', file=C)
    print('#include "src/core/config/load_config.h"', file=C)
    print('#include "absl/types/optional.h"', file=C)
    print('#include "absl/strings/escaping.h"', file=C)
    print('#include "absl/flags/flag.h"', file=C)
    print(file=C)

    for attr in attrs:
        if "prelude" in attr:
            print(attr["prelude"], file=C)

    for attr in attrs:
        print(
            "ABSL_FLAG(%s, %s, {}, %s);"
            % (
                FLAG_TYPE[attr["type"]],
                "grpc_" + attr["name"],
                c_str(attr["description"]),
            ),
            file=C,
        )
    print(file=C)
    print("namespace grpc_core {", file=C)
    print(file=C)
    print("ConfigVars::ConfigVars(const Overrides& overrides) :", file=C)
    initializers = [
        '%s_(LoadConfig(FLAGS_grpc_%s, "GRPC_%s", overrides.%s, %s))'
        % (
            attr["name"],
            attr["name"],
            attr["name"].upper(),
            attr["name"],
            DEFAULT_VALUE[attr["type"]](attr["default"], attr["name"]),
        )
        for attr in attrs_in_packing_order
        if attr.get("force-load-on-access", False) == False
    ]
    initializers += [
        "override_%s_(overrides.%s)" % (attr["name"], attr["name"])
        for attr in attrs_in_packing_order
        if attr.get("force-load-on-access", False)
    ]
    print(",".join(initializers), file=C)
    print("{}", file=C)
    print(file=C)
    for attr in attrs:
        if attr.get("force-load-on-access", False):
            print(
                "%s ConfigVars::%s() const { return LoadConfig(FLAGS_grpc_%s,"
                ' "GRPC_%s", override_%s_, %s); }'
                % (
                    MEMBER_TYPE[attr["type"]],
                    snake_to_pascal(attr["name"]),
                    attr["name"],
                    attr["name"].upper(),
                    attr["name"],
                    DEFAULT_VALUE[attr["type"]](attr["default"], attr["name"]),
                ),
                file=C,
            )
            print(file=C)
    print("std::string ConfigVars::ToString() const {", file=C)
    print("  return absl::StrCat(", file=C)
    for i, attr in enumerate(attrs):
        if i:
            print(",", file=C)
            print(c_str(", " + attr["name"] + ": "), file=C)
        else:
            print(c_str(attr["name"] + ": "), file=C)
        print(
            ",",
            TO_STRING[attr["type"]].replace(
                "$", snake_to_pascal(attr["name"]) + "()"
            ),
            file=C,
        )
    print(");}", file=C)
    print(file=C)
    print("}", file=C)
